from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import TroposphereType

from troposphere import logs, Output, Ref


LOG_RETENTION_VALUES = [
    0, 1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180, 365, 400, 545, 731, 1827,
    3653
]
LOG_RETENTION_STRINGS = [str(x) for x in LOG_RETENTION_VALUES]


def validate_cloudwatch_log_retention(value):
    if value not in LOG_RETENTION_VALUES:
        raise ValueError(
            "%d is not a valid retention period. Must be one of: %s" % (
                value,
                ', '.join(LOG_RETENTION_STRINGS)
            )
        )
    return value


class SubscriptionFilters(Blueprint):

    VARIABLES = {
        "SubscriptionFilters": {
            "type": TroposphereType(logs.SubscriptionFilter, many=True),
            "description": "Subscription filters to create.",
        }
    }

    def create_template(self):
        t = self.template
        variables = self.get_variables()

        for _filter in variables["SubscriptionFilters"]:
            t.add_resource(_filter)
            t.add_output(
                Output(
                    "%sName" % _filter.title,
                    Value=Ref(_filter)
                )
            )



from stacker.blueprints.base import Blueprint

from troposphere import ecr


class Repositories(Blueprint):

    VARIABLES = {
        "Repositories": {
            "type": list,
            "description": "A list of repository names to create."
        }
    }

    def create_template(self):
        t = self.template
        variables = self.get_variables()

        for repo in variables["Repositories"]:
            t.add_resource(
                ecr.Repository(
                    "%sRepository" % repo,
                    RepositoryName=repo,
                )
            )



from troposphere import (
    GetAtt, Ref, ec2, Output, Tags
)
from troposphere.rds import (
    DBSubnetGroup,
    DBClusterParameterGroup,
    DBCluster,
)
from troposphere.route53 import RecordSetType

from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import CFNString

from stacker_blueprints.rds.base import validate_backup_retention_period

# Resource name constants
SUBNET_GROUP = "SubnetGroup"
PARAMETER_GROUP = "ClusterParameterGroup"
SECURITY_GROUP = "SecurityGroup"
DBCLUSTER = "DBCluster"
DNS_RECORD = "DBClusterMasterDnsRecord"


class Cluster(Blueprint):
    VARIABLES = {
        "BackupRetentionPeriod": {
            "type": int,
            "description": "Number of days to retain database backups.",
            "validator": validate_backup_retention_period,
            "default": 7,
        },
        "DatabaseName": {
            "type": str,
            "description": "Initial db to create in database."
        },
        "DBFamily": {
            "type": str,
            "description": "DBFamily for ParameterGroup.",
        },
        "ClusterParameters": {
            "type": dict,
            "description": "A dictionary of parameters to apply to the "
                           "cluster.",
            "default": {},
        },
        "VpcId": {
            "type": str,
            "description": "Vpc Id"
        },
        "Subnets": {
            "type": str,
            "description": "A comma separated list of subnet ids."
        },

        "EngineVersion": {
            "type": str,
            "description": "Database engine version for the RDS Instance.",
        },
        "MasterUser": {
            "type": str,
            "description": "Name of the master user in the db.",
        },
        "MasterUserPassword": {
            "type": CFNString,
            "no_echo": True,
            "description": "Master user password."
        },
        "PreferredBackupWindow": {
            "type": str,
            "description": "A (minimum 30 minute) window in HH:MM-HH:MM "
                           "format in UTC for backups. Default: 4am-5am "
                           "PST",
            "default": "12:00-13:00"
        },
        "PreferredMaintenanceWindow": {
            "type": str,
            "description": "A (minimum 30 minute) window in "
                           "DDD:HH:MM-DDD:HH:MM format in UTC for "
                           "backups. Default: Sunday 3am-4am PST",
            "default": "Sun:11:00-Sun:12:00"
        },
        "SnapshotIdentifier": {
            "type": str,
            "description": "The snapshot you want the db restored from.",
            "default": "",
        },
        "StorageEncrypted": {
            "type": bool,
            "description": "Set to 'false' to disable encrypted storage.",
            "default": True,
        },
        "Tags": {
            "type": dict,
            "description": "An optional dictionary of tags to put on the "
                           "database instance.",
            "default": {}
        },
        "ExistingSecurityGroup": {
            "type": str,
            "description": "The ID of an existing security group to put "
                           "the RDS instance in. If not specified, one "
                           "will be created for you.",
            "default": "",
        },
        "InternalZoneId": {
            "type": str,
            "default": "",
            "description": "Internal zone Id, if you have one."
        },
        "InternalZoneName": {
            "type": str,
            "default": "",
            "description": "Internal zone name, if you have one."
        },
        "InternalHostname": {
            "type": str,
            "default": "",
            "description": "Internal domain name, if you have one."
        },
    }

    def engine(self):
        return None

    def defined_variables(self):
        variables = super(Cluster, self).defined_variables()

        if not self.engine():
            variables['Engine'] = {
                "type": str,
                "description": "Database engine for the Aurora Cluster.",
            }

        return variables

    def should_create_internal_hostname(self):
        variables = self.get_variables()
        return all(
            [
                variables["InternalZoneId"],
                variables["InternalZoneName"],
                variables["InternalHostname"]
            ]
        )

    def get_db_snapshot_identifier(self):
        variables = self.get_variables()
        return variables["DBSnapshotIdentifier"] or Ref("AWS::NoValue")

    def get_tags(self):
        variables = self.get_variables()
        tag_var = variables["Tags"]
        t = {"Name": self.name}
        t.update(tag_var)
        return Tags(**t)

    def create_subnet_group(self):
        t = self.template
        variables = self.get_variables()
        t.add_resource(
            DBSubnetGroup(
                SUBNET_GROUP,
                DBSubnetGroupDescription="%s VPC subnet group." % self.name,
                SubnetIds=variables["Subnets"].split(",")
            )
        )
        t.add_output(Output("SubnetGroup", Value=Ref(SUBNET_GROUP)))

    def create_security_group(self):
        t = self.template
        variables = self.get_variables()
        self.security_group = variables["ExistingSecurityGroup"]
        if not variables["ExistingSecurityGroup"]:
            sg = t.add_resource(
                ec2.SecurityGroup(
                    SECURITY_GROUP,
                    GroupDescription="%s RDS security group" % self.name,
                    VpcId=variables["VpcId"]
                )
            )
            self.security_group = Ref(sg)
        t.add_output(Output("SecurityGroup", Value=self.security_group))

    def get_master_endpoint(self):
        endpoint = GetAtt(DBCLUSTER, "Endpoint.Address")
        return endpoint

    def create_parameter_group(self):
        t = self.template
        variables = self.get_variables()
        params = variables["ClusterParameters"]
        if params:
            t.add_resource(
                DBClusterParameterGroup(
                    PARAMETER_GROUP,
                    Description=self.name,
                    Family=variables["DBFamily"],
                    Parameters=params,
                )
            )

    def create_cluster(self):
        t = self.template
        variables = self.get_variables()
        parameter_group = Ref("AWS::NoValue")
        if variables["ClusterParameters"]:
            parameter_group = Ref(PARAMETER_GROUP)

        t.add_resource(
            DBCluster(
                DBCLUSTER,
                BackupRetentionPeriod=variables["BackupRetentionPeriod"],
                DBClusterParameterGroupName=parameter_group,
                DBSubnetGroupName=Ref(SUBNET_GROUP),
                Engine=self.engine() or variables["Engine"],
                EngineVersion=variables["EngineVersion"],
                MasterUsername=variables["MasterUser"],
                MasterUserPassword=Ref("MasterUserPassword"),
                PreferredBackupWindow=variables["PreferredBackupWindow"],
                PreferredMaintenanceWindow=variables[
                    "PreferredMaintenanceWindow"],
                Tags=self.get_tags(),
                VpcSecurityGroupIds=[self.security_group, ]
            )
        )

    def create_dns_records(self):
        t = self.template
        variables = self.get_variables()

        # Setup CNAME to cluster
        if self.should_create_internal_hostname():
            hostname = "%s.%s" % (
                variables["InternalHostname"],
                variables["InternalZoneName"]
            )
            t.add_resource(
                RecordSetType(
                    DNS_RECORD,
                    # Appends a "." to the end of the domain
                    HostedZoneId=variables["InternalZoneId"],
                    Comment="RDS DB CNAME Record",
                    Name=hostname,
                    Type="CNAME",
                    TTL="120",
                    ResourceRecords=[self.get_master_endpoint()],
                )
            )

    def create_outputs(self):
        t = self.template
        t.add_output(
            Output("MasterEndpoint", Value=self.get_master_endpoint())
        )
        t.add_output(Output("Cluster", Value=Ref(DBCLUSTER)))
        if self.should_create_internal_hostname():
            t.add_output(
                Output("DBCname", Value=Ref(DNS_RECORD))
            )

    def create_template(self):
        self.create_subnet_group()
        self.create_security_group()
        self.create_parameter_group()
        self.create_cluster()
        self.create_dns_records()
        self.create_outputs()


class AuroraCluster(Cluster):
    def engine(self):
        return "aurora"



import logging

from troposphere import Base64, Join

from stacker.blueprints.base import Blueprint

logger = logging.getLogger(__name__)


class EmpireBase(Blueprint):
    def create_conditions(self):
        logger.debug("No conditions to setup for %s", self.name)

    def create_security_groups(self):
        logger.debug("No security_groups to setup for %s", self.name)

    def create_ecs_cluster(self):
        logger.debug("No ecs cluster to setup for %s", self.name)

    def create_load_balancer(self):
        logger.debug("No load_balancer to setup for %s", self.name)

    def create_iam_profile(self):
        logger.debug("No iam_profile to setup for %s", self.name)

    def create_autoscaling_group(self):
        logger.debug("No autoscaling_group to setup for %s", self.name)

    def generate_seed_contents(self):
        raise Exception('Empire subclass must define seed contents')

    def generate_user_data(self):
        contents = Join("", self.generate_seed_contents())
        stanza = Base64(Join(
            "",
            [
                "#cloud-config\n",
                "write_files:\n",
                "  - encoding: b64\n",
                "    content: ", Base64(contents), "\n",
                "    owner: root:root\n",
                "    path: /etc/empire/seed\n",
                "    permissions: 0640\n"
            ]
        ))
        return stanza

    def create_template(self):
        self.create_conditions()
        self.create_security_groups()
        self.create_ecs_cluster()
        self.create_load_balancer()
        self.create_iam_profile()
        self.create_autoscaling_group()



from awacs.aws import (
    Allow,
    Condition,
    Policy,
    Statement,
    StringEquals,
    StringLike,
)

import awacs.logs
import awacs.s3
import awacs.firehose
import awacs.kms
from awacs.helpers.trust import make_simple_assume_statement

from stacker.blueprints.base import Blueprint

from troposphere import (
    iam,
    logs,
    firehose,
    Join,
    GetAtt,
    Output,
    Ref,
)

from ..policies import (
    s3_arn,
    write_to_cloudwatch_logs_stream_statements,
)

from ..cloudwatch_logs import (
    LOG_RETENTION_STRINGS,
    validate_cloudwatch_log_retention,
)

LOG_GROUP = "LogGroup"
S3_LOG_STREAM = "S3LogStream"
ROLE = "Role"

REGION = Ref("AWS::Region")
NOVALUE = Ref("AWS::NoValue")


def make_simple_assume_policy(*principals):
    return Policy(
        Statement=[
            make_simple_assume_statement(*principals)
        ]
    )


def s3_write_statements(bucket_name):
    return [
        Statement(
            Effect=Allow,
            Action=[
                awacs.s3.AbortMultipartUpload,
                awacs.s3.GetBucketLocation,
                awacs.s3.GetObject,
                awacs.s3.ListBucket,
                awacs.s3.ListBucketMultipartUploads,
                awacs.s3.PutObject,
            ],
            Resource=[
                s3_arn(bucket_name),
                s3_arn(Join("/", [bucket_name, "*"]))
            ],
        ),
    ]


def kms_key_statements(key_arn, bucket_arn, bucket_prefix):
    s3_endpoint = Join(
        '',
        [
            "s3.", REGION, "amazonaws.com"
        ]
    )
    return [
        Statement(
            Effect=Allow,
            Action=[
                awacs.kms.Decrypt,
                awacs.kms.GenerateDataKey,
            ],
            Resource=[key_arn],
            Condition=Condition(
                [
                    StringEquals(
                        "kms:ViaService", s3_endpoint
                    ),
                    StringLike(
                        "kms:EncryptionContext:aws:s3:arn",
                        Join('', [bucket_arn, bucket_prefix, "*"])
                    )
                ]
            )

        )
    ]


class BaseDeliveryStream(Blueprint):
    VARIABLES = {
        "BucketName": {
            "type": str,
            "description": "Name of existing bucket to stream firehose "
                           "data to."
        },
        "S3Prefix": {
            "type": str,
            "description": "The prefix used when writing objects in the s3 "
                           "bucket.",
            "default": "/",
        },
        "EncryptionKeyArn": {
            "type": str,
            "description": "ARN of the KMS key to use to encrypt objects in "
                           "the s3 bucket.",
            "default": "",
        },
        "BufferingHints": {
            "type": dict,
            "description": "A dictionary with buffering hints for writing "
                           "objects to the s3 bucket. Valid keys are: "
                           "IntervalInSeconds, SizeInMBs",
            "default": {"IntervalInSeconds": 300, "SizeInMBs": 5},
        },
        "CompressionFormat": {
            "type": str,
            "description": "The compression format used by the Firehose when "
                           "writing objects in the s3 bucket.",
            "default": "UNCOMPRESSED",
        },
        "LogRetentionDays": {
            "type": int,
            "description": "Time in days to retain Cloudwatch Logs. Accepted "
                           "values: %s. Default 0 - retain forever." % (
                               ', '.join(LOG_RETENTION_STRINGS)),
            "default": 0,
            "validator": validate_cloudwatch_log_retention,
        }
    }

    def buffering_hints(self):
        hints_config = self.get_variables()["BufferingHints"]
        return firehose.BufferingHints(**hints_config)

    def encryption_config(self):
        key_arn = self.get_variables()["EncryptionKeyArn"]
        if key_arn:
            return firehose.EncryptionConfiguration(
                KMSEncryptionConfig=firehose.KMSEncryptionConfig(
                    AWSKMSKeyARN=key_arn
                )
            )
        else:
            return NOVALUE

    def s3_bucket_arn(self):
        bucket_name = self.get_variables()["BucketName"]
        return s3_arn(bucket_name)

    def cloudwatch_logging_options(self, log_group, log_stream):
        return firehose.CloudWatchLoggingOptions(
            Enabled=True,
            LogGroupName=Ref(log_group),
            LogStreamName=Ref(log_stream),
        )

    def s3_destination_config_dict(self):
        t = self.template
        variables = self.get_variables()

        t.add_output(Output("BucketName", Value=variables["BucketName"]))

        return {
            "BucketARN": self.s3_bucket_arn(),
            "RoleARN": GetAtt(self.role, "Arn"),
            "CompressionFormat": variables['CompressionFormat'],
            "BufferingHints": self.buffering_hints(),
            "Prefix": variables["S3Prefix"],
            "EncryptionConfiguration": self.encryption_config(),
            "CloudWatchLoggingOptions": self.cloudwatch_logging_options(
                self.log_group,
                self.s3_log_stream
            )
        }

    def generate_iam_policy_statements(self):
        variables = self.get_variables()
        bucket_name = variables["BucketName"]
        bucket_arn = self.s3_bucket_arn()
        s3_prefix = variables["S3Prefix"]
        key_arn = variables["EncryptionKeyArn"]

        statements = []
        statements.extend(s3_write_statements(bucket_name))
        statements.extend(
            write_to_cloudwatch_logs_stream_statements(
                Ref(self.log_group), Ref(self.s3_log_stream)
            )
        )

        if key_arn:
            statements.extend(
                kms_key_statements(
                    key_arn, bucket_arn, s3_prefix
                )
            )

        return statements

    def generate_iam_policy(self):
        name_prefix = self.context.get_fqn(self.name)
        return iam.Policy(
            PolicyName="{}-policy".format(name_prefix),
            PolicyDocument=Policy(
                Statement=self.generate_iam_policy_statements()
            )
        )

    def create_role(self):
        t = self.template

        self.role = t.add_resource(
            iam.Role(
                ROLE,
                AssumeRolePolicyDocument=make_simple_assume_policy(
                    "firehose.amazonaws.com"
                ),
                Path="/",
                Policies=[self.generate_iam_policy()]
            )
        )

        t.add_output(Output("RoleName", Value=Ref(self.role)))
        t.add_output(Output("RoleArn", Value=GetAtt(self.role, "Arn")))

    def create_log_group(self):
        t = self.template
        variables = self.get_variables()
        log_retention = variables["LogRetentionDays"] or NOVALUE

        self.log_group = t.add_resource(
            logs.LogGroup(
                LOG_GROUP,
                RetentionInDays=log_retention,
            )
        )

        t.add_output(Output("LogGroupName", Value=Ref(self.log_group)))
        t.add_output(
            Output("LogGroupArn", Value=GetAtt(self.log_group, "Arn"))
        )

    def create_s3_log_stream(self):
        t = self.template

        self.s3_log_stream = t.add_resource(
            logs.LogStream(
                S3_LOG_STREAM,
                LogGroupName=Ref(self.log_group),
                DependsOn=self.log_group.title
            )
        )

        t.add_output(Output("S3LogStreamName", Value=Ref(self.s3_log_stream)))

    def create_log_stream(self):
        self.create_s3_log_stream()

    def create_delivery_stream(self):
        raise NotImplementedError("create_delivery_stream must be implemented "
                                  "by a subclass.")

    def create_delivery_stream_output(self):
        t = self.template
        t.add_output(
            Output("DeliveryStreamName", Value=Ref(self.delivery_stream))
        )

    def create_template(self):
        self.create_log_group()
        self.create_log_stream()
        self.create_role()
        self.create_delivery_stream()
        self.create_delivery_stream_output()



from troposphere import (
    GetAtt,
    Join,
    Output,
    Ref,
    iam,
    logs,
    ec2,
)

from troposphere.iam import Policy as TropoPolicy

from stacker.blueprints.base import Blueprint

from awacs.aws import (
    Statement,
    Policy,
)

import awacs
import awacs.logs

from .policies import flowlogs_assumerole_policy
from .cloudwatch_logs import (
    LOG_RETENTION_STRINGS,
    validate_cloudwatch_log_retention
)

ALLOWED_TRAFFIC_TYPES = ["ACCEPT", "REJECT", "ALL"]
JOINED_TRAFFIC_TYPES = '/'.join(ALLOWED_TRAFFIC_TYPES)
LOG_RETENTION_DEFAULT = 0
CLOUDWATCH_ROLE_NAME = "Role"
FLOW_LOG_GROUP_NAME = "LogGroup"
FLOW_LOG_STREAM_NAME = "LogStream"


def vpc_flow_log_cloudwatch_policy(log_group_arn):
    return Policy(
        Statement=[
            Statement(
                Effect="Allow",
                Action=[
                    awacs.logs.DescribeLogGroups
                ],
                Resource=["*"],
            ),
            Statement(
                Effect="Allow",
                Action=[
                    awacs.logs.CreateLogStream,
                    awacs.logs.DescribeLogStreams,
                    awacs.logs.PutLogEvents,
                ],
                Resource=[
                    log_group_arn,
                    Join('', [log_group_arn, ":*"]),
                ],
            ),
        ]
    )


def validate_traffic_type(traffic_type):
    if traffic_type not in ALLOWED_TRAFFIC_TYPES:
        raise ValueError(
            "Traffic type must be one of the following: " +
            "%s" % JOINED_TRAFFIC_TYPES
        )

    return traffic_type


class FlowLogs(Blueprint):
    VARIABLES = {
        "Retention": {
            "type": int,
            "description": "Time in days to retain Cloudwatch Logs. Accepted "
                           "values: %s. Default 0 - retain forever." % (
                               ', '.join(LOG_RETENTION_STRINGS)),
            "default": LOG_RETENTION_DEFAULT,
            "validator": validate_cloudwatch_log_retention,

        },
        "VpcId": {
            "type": str,
            "description": "ID of the VPC that flow logs will be enabled "
                           "for.",
        },
        "TrafficType": {
            "type": str,
            "description": "Type of traffic to log. Must be one of the "
                           "following: %s" % JOINED_TRAFFIC_TYPES,
            "validator": validate_traffic_type,
            "default": "ALL",
        },
    }

    def create_template(self):
        t = self.template
        variables = self.get_variables()

        self.log_group = t.add_resource(
            logs.LogGroup(
                FLOW_LOG_GROUP_NAME,
                RetentionInDays=variables["Retention"],
            )
        )

        t.add_output(
            Output(
                "%sName" % FLOW_LOG_GROUP_NAME,
                Value=Ref(self.log_group)
            )
        )
        t.add_output(
            Output(
                "%sArn" % FLOW_LOG_GROUP_NAME,
                Value=GetAtt(self.log_group, "Arn")
            )
        )

        self.role = t.add_resource(
            iam.Role(
                CLOUDWATCH_ROLE_NAME,
                AssumeRolePolicyDocument=flowlogs_assumerole_policy(),
                Path="/",
                Policies=[
                    TropoPolicy(
                        PolicyName="vpc_cloudwatch_flowlog_policy",
                        PolicyDocument=vpc_flow_log_cloudwatch_policy(
                            GetAtt(self.log_group, "Arn")
                        ),
                    ),
                ]
            )
        )

        t.add_output(
            Output(
                "%sName" % CLOUDWATCH_ROLE_NAME,
                Value=Ref(self.role)
            )
        )
        role_arn = GetAtt(self.role, "Arn")
        t.add_output(
            Output(
                "%sArn" % CLOUDWATCH_ROLE_NAME,
                Value=role_arn
            )
        )

        self.log_stream = t.add_resource(
            ec2.FlowLog(
                FLOW_LOG_STREAM_NAME,
                DeliverLogsPermissionArn=role_arn,
                LogGroupName=Ref(FLOW_LOG_GROUP_NAME),
                ResourceId=variables["VpcId"],
                ResourceType="VPC",
                TrafficType=variables["TrafficType"],
            )
        )

        t.add_output(
            Output(
                "%sName" % FLOW_LOG_STREAM_NAME,
                Value=Ref(self.log_stream)
            )
        )



from awacs.aws import (
    Action,
    Allow,
    Policy,
    Principal,
    Statement,
)

from troposphere import Join, Ref

from awacs import sts, s3, logs

ACCOUNT_ID = Ref("AWS::AccountId")
REGION = Ref("AWS::Region")


def make_simple_assume_statement(*principals):
    return Statement(
        Principal=Principal('Service', principals),
        Effect=Allow,
        Action=[sts.AssumeRole])


def make_simple_assume_policy(*principals):
    return Policy(
        Statement=[
            make_simple_assume_statement(*principals)])


def s3_arn(bucket):
    return Join('', ['arn:aws:s3:::', bucket])


def read_only_s3_bucket_policy_statements(buckets):
    """ Read only policy an s3 bucket. """
    list_buckets = [s3_arn(b) for b in buckets]
    object_buckets = [s3_arn(Join("/", [b, "*"])) for b in buckets]

    bucket_resources = list_buckets + object_buckets

    return [
        Statement(
            Effect=Allow,
            Resource=[s3_arn("*")],
            Action=[s3.ListAllMyBuckets]
        ),
        Statement(
            Effect=Allow,
            Resource=bucket_resources,
            Action=[Action('s3', 'Get*'), Action('s3', 'List*')]
        )
    ]


def read_only_s3_bucket_policy(buckets):
    return Policy(Statement=read_only_s3_bucket_policy_statements(buckets))


def read_write_s3_bucket_policy_statements(buckets):
    list_buckets = [s3_arn(b) for b in buckets]
    object_buckets = [s3_arn(Join("/", [b, "*"])) for b in buckets]
    return [
        Statement(
            Effect="Allow",
            Action=[
                s3.GetBucketLocation,
                s3.ListAllMyBuckets,
            ],
            Resource=["arn:aws:s3:::*"]
        ),
        Statement(
            Effect=Allow,
            Action=[
                s3.ListBucket,
            ],
            Resource=list_buckets,
        ),
        Statement(
            Effect=Allow,
            Action=[
                s3.GetObject,
                s3.PutObject,
                s3.PutObjectAcl,
                s3.DeleteObject,
            ],
            Resource=object_buckets,
        ),
    ]


def read_write_s3_bucket_policy(buckets):
    return Policy(Statement=read_write_s3_bucket_policy_statements(buckets))


def log_stream_arn(log_group_name, log_stream_name):
    return Join(
        '',
        [
            "arn:aws:logs:", REGION, ":", ACCOUNT_ID, ":log-group:",
            log_group_name, ":log-stream:", log_stream_name
        ]
    )


def write_to_cloudwatch_logs_stream_statements(log_group_name,
                                               log_stream_name):
    return [
        Statement(
            Effect=Allow,
            Action=[logs.PutLogEvents],
            Resource=[log_stream_arn(log_group_name, log_stream_name)]
        )
    ]


def write_to_cloudwatch_logs_stream_policy(log_group_name, log_stream_name):
    return Policy(
        Statement=write_to_cloudwatch_logs_stream_statements(log_group_name,
                                                             log_stream_name)
    )


def flowlogs_assumerole_policy():
    return make_simple_assume_policy("vpc-flow-logs.amazonaws.com")



import copy

from troposphere import (
    Ref, FindInMap, Not, Equals, And, Condition, Join, ec2, autoscaling,
    If, GetAtt
)
from troposphere import elasticloadbalancing as elb
from troposphere.autoscaling import Tag as ASTag
from troposphere.route53 import RecordSetType

from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import (
    CFNCommaDelimitedList,
    CFNNumber,
    CFNString,
    EC2VPCId,
    EC2KeyPairKeyName,
    EC2SecurityGroupId,
    EC2SubnetIdList,
)

CLUSTER_SG_NAME = "%sSG"
ELB_SG_NAME = "%sElbSG"
ELB_NAME = "%sLoadBalancer"


class AutoscalingGroup(Blueprint):
    VARIABLES = {
        'VpcId': {'type': EC2VPCId, 'description': 'Vpc Id'},
        'DefaultSG': {'type': EC2SecurityGroupId,
                      'description': 'Top level security group.'},
        'BaseDomain': {
            'type': CFNString,
            'default': '',
            'description': 'Base domain for the stack.'},
        'PrivateSubnets': {'type': EC2SubnetIdList,
                           'description': 'Subnets to deploy private '
                                          'instances in.'},
        'PublicSubnets': {'type': EC2SubnetIdList,
                          'description': 'Subnets to deploy public (elb) '
                                         'instances in.'},
        'AvailabilityZones': {'type': CFNCommaDelimitedList,
                              'description': 'Availability Zones to deploy '
                                             'instances in.'},
        'InstanceType': {'type': CFNString,
                         'description': 'EC2 Instance Type',
                         'default': 'm3.medium'},
        'MinSize': {'type': CFNNumber,
                    'description': 'Minimum # of instances.',
                    'default': '1'},
        'MaxSize': {'type': CFNNumber,
                    'description': 'Maximum # of instances.',
                    'default': '5'},
        'SshKeyName': {'type': EC2KeyPairKeyName},
        'ImageName': {
            'type': CFNString,
            'description': 'The image name to use from the AMIMap (usually '
                           'found in the config file.)'},
        'ELBHostName': {
            'type': CFNString,
            'description': 'A hostname to give to the ELB. If not given '
                           'no ELB will be created.',
            'default': ''},
        'ELBCertName': {
            'type': CFNString,
            'description': 'The SSL certificate name to use on the ELB.',
            'default': ''},
        'ELBCertType': {
            'type': CFNString,
            'description': 'The SSL certificate type to use on the ELB.',
            'default': ''},
    }

    def create_conditions(self):
        self.template.add_condition(
            "CreateELB",
            Not(Equals(Ref("ELBHostName"), "")))
        self.template.add_condition(
            "SetupDNS",
            Not(Equals(Ref("BaseDomain"), "")))
        self.template.add_condition(
            "UseSSL",
            Not(Equals(Ref("ELBCertName"), "")))
        self.template.add_condition(
            "CreateSSLELB",
            And(Condition("CreateELB"), Condition("UseSSL")))
        self.template.add_condition(
            "SetupELBDNS",
            And(Condition("CreateELB"), Condition("SetupDNS")))
        self.template.add_condition(
            "UseIAMCert",
            Not(Equals(Ref("ELBCertType"), "acm")))

    def create_security_groups(self):
        t = self.template
        asg_sg = CLUSTER_SG_NAME % self.name
        elb_sg = ELB_SG_NAME % self.name
        t.add_resource(ec2.SecurityGroup(
            asg_sg,
            GroupDescription=asg_sg,
            VpcId=Ref("VpcId")))
        # ELB Security group, if ELB is used
        t.add_resource(
            ec2.SecurityGroup(
                elb_sg,
                GroupDescription=elb_sg,
                VpcId=Ref("VpcId"),
                Condition="CreateELB"))
        # Add SG rules here
        # Allow ELB to connect to ASG on port 80
        t.add_resource(ec2.SecurityGroupIngress(
            "%sElbToASGPort80" % self.name,
            IpProtocol="tcp", FromPort="80", ToPort="80",
            SourceSecurityGroupId=Ref(elb_sg),
            GroupId=Ref(asg_sg),
            Condition="CreateELB"))
        # Allow Internet to connect to ELB on port 80
        t.add_resource(ec2.SecurityGroupIngress(
            "InternetTo%sElbPort80" % self.name,
            IpProtocol="tcp", FromPort="80", ToPort="80",
            CidrIp="0.0.0.0/0",
            GroupId=Ref(elb_sg),
            Condition="CreateELB"))
        t.add_resource(ec2.SecurityGroupIngress(
            "InternetTo%sElbPort443" % self.name,
            IpProtocol="tcp", FromPort="443", ToPort="443",
            CidrIp="0.0.0.0/0",
            GroupId=Ref(elb_sg),
            Condition="CreateSSLELB"))

    def setup_listeners(self):
        no_ssl = [elb.Listener(
            LoadBalancerPort=80,
            Protocol='HTTP',
            InstancePort=80,
            InstanceProtocol='HTTP'
        )]

        # Choose proper certificate source
        acm_cert = Join("", [
            "arn:aws:acm:", Ref("AWS::Region"), ":", Ref("AWS::AccountId"),
            ":certificate/", Ref("ELBCertName")])
        iam_cert = Join("", [
            "arn:aws:iam::", Ref("AWS::AccountId"), ":server-certificate/",
            Ref("ELBCertName")])
        cert_id = If("UseIAMCert", iam_cert, acm_cert)

        with_ssl = copy.deepcopy(no_ssl)
        with_ssl.append(elb.Listener(
            LoadBalancerPort=443,
            InstancePort=80,
            Protocol='HTTPS',
            InstanceProtocol="HTTP",
            SSLCertificateId=cert_id))
        listeners = If("UseSSL", with_ssl, no_ssl)

        return listeners

    def create_load_balancer(self):
        t = self.template
        elb_name = ELB_NAME % self.name
        elb_sg = ELB_SG_NAME % self.name
        t.add_resource(elb.LoadBalancer(
            elb_name,
            HealthCheck=elb.HealthCheck(
                Target='HTTP:80/',
                HealthyThreshold=3,
                UnhealthyThreshold=3,
                Interval=5,
                Timeout=3),
            Listeners=self.setup_listeners(),
            SecurityGroups=[Ref(elb_sg), ],
            Subnets=Ref("PublicSubnets"),
            Condition="CreateELB"))

        # Setup ELB DNS
        t.add_resource(
            RecordSetType(
                '%sDnsRecord' % elb_name,
                # Appends a '.' to the end of the domain
                HostedZoneName=Join("", [Ref("BaseDomain"), "."]),
                Comment='Router ELB DNS',
                Name=Join('.', [Ref("ELBHostName"), Ref("BaseDomain")]),
                Type='CNAME',
                TTL='120',
                ResourceRecords=[
                    GetAtt(elb_name, 'DNSName')],
                Condition="SetupELBDNS"))

    def get_launch_configuration_parameters(self):
        return {
            'ImageId': FindInMap('AmiMap', Ref("AWS::Region"),
                                 Ref('ImageName')),
            'InstanceType': Ref("InstanceType"),
            'KeyName': Ref("SshKeyName"),
            'SecurityGroups': self.get_launch_configuration_security_groups(),
        }

    def get_autoscaling_group_parameters(self, launch_config_name, elb_name):
        return {
            'AvailabilityZones': Ref("AvailabilityZones"),
            'LaunchConfigurationName': Ref(launch_config_name),
            'MinSize': Ref("MinSize"),
            'MaxSize': Ref("MaxSize"),
            'VPCZoneIdentifier': Ref("PrivateSubnets"),
            'LoadBalancerNames': If("CreateELB", [Ref(elb_name), ], []),
            'Tags': [ASTag('Name', self.name, True)],
        }

    def get_launch_configuration_security_groups(self):
        sg_name = CLUSTER_SG_NAME % self.name
        return [Ref("DefaultSG"), Ref(sg_name)]

    def create_autoscaling_group(self):
        name = "%sASG" % self.name
        launch_config = "%sLaunchConfig" % name
        elb_name = ELB_NAME % self.name
        t = self.template
        t.add_resource(autoscaling.LaunchConfiguration(
            launch_config,
            **self.get_launch_configuration_parameters()
        ))
        t.add_resource(autoscaling.AutoScalingGroup(
            name,
            **self.get_autoscaling_group_parameters(launch_config, elb_name)
        ))

    def create_template(self):
        self.create_conditions()
        self.create_security_groups()
        self.create_load_balancer()
        self.create_autoscaling_group()



import unittest

from stacker.blueprints.testutil import BlueprintTestCase
from stacker.context import Context
from stacker.variables import Variable

from stacker_blueprints.generic import GenericResourceCreator


class TestGenericResourceCreator(BlueprintTestCase):
    def setUp(self):
        self.ctx = Context({'namespace': 'test'})

    def test_create_template(self):
        blueprint = GenericResourceCreator('test_generic_GenericResourceCreator', self.ctx)
        blueprint.resolve_variables(
            [
                Variable('Class', 'ec2.Volume'),
                Variable('Output', 'VolumeId'),
                Variable('Properties', {
                    'VolumeType': 'gp2',
                    'Size': '600',
                    'Encrypted': 'true',
                    'AvailabilityZone': 'us-east-1b',
                }),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)


if __name__ == '__main__':
    unittest.main()



import logging

from awacs import (
    awslambda,
    ecs,
    ec2,
    events,
    iam,
    route53,
    kinesis,
    sns,
    logs,
    sqs,
    s3,
    cloudformation,
    elasticloadbalancing as elb,
    ecr,
)
from awacs.aws import (
    Statement,
    Allow,
    Policy,
    Action,
    Principal,
    Condition,
    SourceArn,
    ArnEquals,
)
from troposphere import (
    Ref,
    Join,
)

logger = logging.getLogger(__name__)


def ecs_agent_policy():
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[
                    ecs.CreateCluster,
                    ecs.RegisterContainerInstance,
                    ecs.DeregisterContainerInstance,
                    ecs.DiscoverPollEndpoint,
                    ecs.Action("Submit*"),
                    ecs.Poll,
                    ecs.Action("StartTelemetrySession")]),
            Statement(
                Effect=Allow,
                Action=[
                    ecr.GetAuthorizationToken,
                    ecr.BatchCheckLayerAvailability,
                    ecr.GetDownloadUrlForLayer,
                    ecr.BatchGetImage,
                ],
                Resource=["*"],
            ),
        ]
    )

    return p


def service_role_policy():
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[
                    ec2.AuthorizeSecurityGroupIngress,
                    Action("ec2", "Describe*"),
                    elb.DeregisterInstancesFromLoadBalancer,
                    Action("elasticloadbalancing", "Describe*"),
                    elb.RegisterInstancesWithLoadBalancer,
                    elb.Action("RegisterTargets"),
                    elb.Action("DeregisterTargets"),
                ]
            )
        ]
    )
    return p


def empire_policy(resources):
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Resource=[resources['CustomResourcesTopic']],
                Action=[sns.Publish]),
            Statement(
                Effect=Allow,
                Resource=[resources['CustomResourcesQueue']],
                Action=[
                    sqs.ReceiveMessage,
                    sqs.DeleteMessage,
                    sqs.ChangeMessageVisibility
                ]),
            Statement(
                Effect=Allow,
                Resource=[resources['TemplateBucket']],
                Action=[
                    s3.PutObject,
                    s3.PutObjectAcl,
                    s3.PutObjectVersionAcl,
                    s3.GetObject,
                    s3.GetObjectVersion,
                    s3.GetObjectAcl,
                    s3.GetObjectVersionAcl]),
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[
                    awslambda.CreateFunction,
                    awslambda.DeleteFunction,
                    awslambda.UpdateFunctionCode,
                    awslambda.GetFunctionConfiguration,
                    awslambda.AddPermission,
                    awslambda.RemovePermission]),
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[
                    events.PutRule,
                    events.DeleteRule,
                    events.DescribeRule,
                    events.EnableRule,
                    events.DisableRule,
                    events.PutTargets,
                    events.RemoveTargets]),
            Statement(
                Effect=Allow,
                Resource=[
                    Join('', [
                        'arn:aws:cloudformation:', Ref('AWS::Region'), ':',
                        Ref('AWS::AccountId'), ':stack/',
                        resources['Environment'], '-*'])],
                Action=[
                    cloudformation.CreateStack,
                    cloudformation.UpdateStack,
                    cloudformation.DeleteStack,
                    cloudformation.ListStackResources,
                    cloudformation.DescribeStackResource,
                    cloudformation.DescribeStacks]),
            Statement(
                Effect=Allow,
                Resource=['*'],
                Action=[cloudformation.ValidateTemplate]),
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[ecs.CreateService, ecs.DeleteService,
                        ecs.DeregisterTaskDefinition,
                        ecs.Action("Describe*"), ecs.Action("List*"),
                        ecs.RegisterTaskDefinition, ecs.RunTask,
                        ecs.StartTask, ecs.StopTask, ecs.SubmitTaskStateChange,
                        ecs.UpdateService]),
            Statement(
                Effect=Allow,
                # TODO: Limit to specific ELB?
                Resource=["*"],
                Action=[
                    elb.Action("Describe*"),
                    elb.AddTags,
                    elb.CreateLoadBalancer,
                    elb.CreateLoadBalancerListeners,
                    elb.DescribeTags,
                    elb.DeleteLoadBalancer,
                    elb.ConfigureHealthCheck,
                    elb.ModifyLoadBalancerAttributes,
                    elb.SetLoadBalancerListenerSSLCertificate,
                    elb.SetLoadBalancerPoliciesOfListener,
                    elb.Action("CreateTargetGroup"),
                    elb.Action("CreateListener"),
                    elb.Action("DeleteListener"),
                    elb.Action("DeleteTargetGroup"),
                    elb.Action("ModifyTargetGroup"),
                    elb.Action("ModifyTargetGroupAttributes"),
                ]
            ),
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[ec2.DescribeSubnets, ec2.DescribeSecurityGroups]
            ),
            Statement(
                Effect=Allow,
                Action=[iam.GetServerCertificate, iam.UploadServerCertificate,
                        iam.DeleteServerCertificate, iam.PassRole],
                Resource=["*"]
            ),
            Statement(
                Effect=Allow,
                Action=[
                    Action("route53", "ListHostedZonesByName"),
                    route53.ChangeResourceRecordSets,
                    route53.ListHostedZones,
                    route53.GetHostedZone,
                    route53.GetChange,
                ],
                # TODO: Limit to specific zones
                Resource=["*"]
            ),
            Statement(
                Effect=Allow,
                Action=[
                    kinesis.DescribeStream,
                    Action(kinesis.prefix, "Get*"),
                    Action(kinesis.prefix, "List*"),
                    kinesis.PutRecord,
                ],
                Resource=["*"]
            ),
            Statement(
                Effect=Allow,
                Action=[
                    ecr.GetAuthorizationToken,
                    ecr.BatchCheckLayerAvailability,
                    ecr.GetDownloadUrlForLayer,
                    ecr.BatchGetImage,
                ],
                Resource=["*"],
            ),
        ]
    )
    return p


def sns_events_policy(topic_arn):
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Action=[sns.Publish],
                Resource=[topic_arn],
            )])

    return p


def logstream_policy():
    """Policy needed for logspout -> kinesis log streaming."""
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Resource=["*"],
                Action=[
                    kinesis.CreateStream, kinesis.DescribeStream,
                    Action(kinesis.prefix, "AddTagsToStream"),
                    Action(kinesis.prefix, "PutRecords")
                ])])
    return p


def runlogs_policy(log_group_ref):
    """Policy needed for Empire -> Cloudwatch logs to record run output."""
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Resource=[
                    Join('', [
                        'arn:aws:logs:*:*:log-group:',
                        log_group_ref,
                        ':log-stream:*'])],
                Action=[
                    logs.CreateLogStream,
                    logs.PutLogEvents,
                ])])
    return p


def sns_to_sqs_policy(topic):
    p = Policy(
        Statement=[
            Statement(
                Effect=Allow,
                Principal=Principal('*'),
                Action=[sqs.SendMessage],
                Resource=["*"],
                Condition=Condition(ArnEquals(SourceArn, topic)))])
    return p



from stacker.blueprints.base import Blueprint

from troposphere import (
    sqs,
    Ref,
    GetAtt,
    Output,
)

from . import util


def validate_queue(queue):
    sqs_queue_properties = [
        "DelaySeconds",
        "FifoQueue",
        "MaximumMessageSize",
        "MessageRetentionPeriod",
        "ReceiveMessageWaitTimeSeconds",
        "RedrivePolicy",
        "VisibilityTimeout",
    ]

    util.check_properties(queue, sqs_queue_properties, "SQS")

    if "RedrivePolicy" in queue:
        queue["RedrivePolicy"] = sqs.RedrivePolicy(**queue["RedrivePolicy"])

    if queue.get("FifoQueue"):
        if not queue.get("QueueName", "").endswith(".fifo"):
            raise ValueError(
                "FIFO queues need to provide a QueueName "
                "that ends with '.fifo'"
            )

    return queue


def validate_queues(queues):
    validated_queues = {}
    for queue_name, queue_config in queues.iteritems():
        validated_queues[queue_name] = validate_queue(queue_config)

    return validated_queues


class Queues(Blueprint):
    """Manages the creation of SQS queues."""

    VARIABLES = {
        "Queues": {
            "type": dict,
            "description": "Dictionary of SQS queue definitions",
            "validator": validate_queues,
        },
    }

    def create_template(self):
        variables = self.get_variables()

        for name, queue_config in variables["Queues"].iteritems():
            self.create_queue(name, queue_config)

    def create_queue(self, queue_name, queue_config):
        t = self.template

        t.add_resource(
            sqs.Queue(
                queue_name,
                **queue_config
            )
        )

        t.add_output(
            Output(queue_name + "Arn", Value=GetAtt(queue_name, "Arn"))
        )
        t.add_output(Output(queue_name + "Url", Value=Ref(queue_name)))



import unittest

from stacker.blueprints.testutil import BlueprintTestCase
from stacker.context import Context
from stacker.variables import Variable

from stacker_blueprints.aws_lambda import Function

from troposphere.awslambda import Code


class TestFunction(BlueprintTestCase):
    def setUp(self):
        self.ctx = Context({'namespace': 'test'})

    def test_create_template(self):
        blueprint = Function('test_aws_lambda_Function', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "Code",
                    Code(S3Bucket="test_bucket", S3Key="code_key")
                ),
                Variable("Description", "Test function."),
                Variable("Environment", {"TEST_NAME": "test_value"}),
                Variable("Runtime", "python2.7"),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)


if __name__ == '__main__':
    unittest.main()



from troposphere import (
    Ref,
    Output,
    GetAtt,
    FindInMap,
)
from troposphere import (
    ec2,
    autoscaling,
    ecs,
)
from troposphere.autoscaling import Tag as ASTag
from troposphere.iam import (
    InstanceProfile,
    Policy,
    Role,
)

from awacs.helpers.trust import (
    get_default_assumerole_policy,
)

from stacker.blueprints.variables.types import (
    CFNCommaDelimitedList,
    CFNNumber,
    CFNString,
    EC2KeyPairKeyName,
    EC2SecurityGroupId,
    EC2SubnetIdList,
    EC2VPCId,
)

from .base import EmpireBase

from .policies import ecs_agent_policy

CLUSTER_SG_NAME = "EmpireControllerSecurityGroup"


class EmpireController(EmpireBase):
    VARIABLES = {
        "VpcId": {
            "type": EC2VPCId,
            "description": "Vpc Id"},
        "DefaultSG": {
            "type": EC2SecurityGroupId,
            "description": "Top level security group."},
        "PrivateSubnets": {
            "type": EC2SubnetIdList,
            "description": "Subnets to deploy private instances in."},
        "AvailabilityZones": {
            "type": CFNCommaDelimitedList,
            "description": "Availability Zones to deploy instances in."},
        "InstanceType": {
            "type": CFNString,
            "description": "Empire AWS Instance Type",
            "default": "m3.medium"},
        "MinHosts": {
            "type": CFNNumber,
            "description": "Minimum # of empire minion instances.",
            "default": "2"},
        "MaxHosts": {
            "type": CFNNumber,
            "description": "Maximum # of empire minion instances.",
            "default": "3"},
        "SshKeyName": {
            "type": EC2KeyPairKeyName},
        "ImageName": {
            "type": CFNString,
            "description": (
                "The image name to use from the AMIMap (usually found in the "
                "config file.)"
            ),
            "default": "empire"},
        "DatabaseSecurityGroup": {
            "type": EC2SecurityGroupId,
            "description": "Security group of Empire database."},
        "DockerRegistry": {
            "type": CFNString,
            "description": (
                "Optional docker registry where private images are located."
            ),
            "default": "https://index.docker.io/v1/"},
        "DockerRegistryUser": {
            "type": CFNString,
            "description": "User for authentication with docker registry."},
        "DockerRegistryPassword": {
            "type": CFNString,
            "no_echo": True,
            "description": (
                "Password for authentication with docker registry."
            )},
        "DockerRegistryEmail": {
            "type": CFNString,
            "description": "Email for authentication with docker registry."},
    }

    def create_security_groups(self):
        t = self.template

        t.add_resource(
            ec2.SecurityGroup(
                CLUSTER_SG_NAME,
                GroupDescription=CLUSTER_SG_NAME,
                VpcId=Ref("VpcId")))

        t.add_output(
            Output("SecurityGroup", Value=Ref(CLUSTER_SG_NAME)))

        # Allow access to the DB
        t.add_resource(
            ec2.SecurityGroupIngress(
                "EmpireControllerDBAccess",
                IpProtocol="tcp", FromPort=5432, ToPort=5432,
                SourceSecurityGroupId=Ref(CLUSTER_SG_NAME),
                GroupId=Ref("DatabaseSecurityGroup")))

    def create_ecs_cluster(self):
        t = self.template
        t.add_resource(ecs.Cluster("EmpireControllerCluster"))
        t.add_output(
            Output("ECSCluster", Value=Ref("EmpireControllerCluster")))

    def build_block_device(self):
        volume = autoscaling.EBSBlockDevice(VolumeSize="50")
        return [autoscaling.BlockDeviceMapping(
            DeviceName="/dev/sdh", Ebs=volume)]

    def generate_iam_policies(self):
        return [
            Policy(
                PolicyName="ecs-agent",
                PolicyDocument=ecs_agent_policy(),
            )]

    def create_iam_profile(self):
        t = self.template
        # Role for Empire Controllers
        t.add_resource(
            Role(
                "EmpireControllerRole",
                AssumeRolePolicyDocument=get_default_assumerole_policy(),
                Path="/",
                Policies=self.generate_iam_policies()))

        t.add_resource(
            InstanceProfile(
                "EmpireControllerProfile",
                Path="/",
                Roles=[Ref("EmpireControllerRole")]))
        t.add_output(Output("IAMRole", Value=Ref("EmpireControllerRole")))

    def generate_seed_contents(self):
        seed = [
            "EMPIRE_HOSTGROUP=controller\n",
            "ECS_CLUSTER=", Ref("EmpireControllerCluster"), "\n",
            "DOCKER_REGISTRY=", Ref("DockerRegistry"), "\n",
            "DOCKER_USER=", Ref("DockerRegistryUser"), "\n",
            "DOCKER_PASS=", Ref("DockerRegistryPassword"), "\n",
            "DOCKER_EMAIL=", Ref("DockerRegistryEmail"), "\n",
        ]
        return seed

    def create_autoscaling_group(self):
        t = self.template
        t.add_resource(
            autoscaling.LaunchConfiguration(
                "EmpireControllerLaunchConfig",
                IamInstanceProfile=GetAtt("EmpireControllerProfile", "Arn"),
                ImageId=FindInMap(
                    "AmiMap",
                    Ref("AWS::Region"),
                    Ref("ImageName")),
                BlockDeviceMappings=self.build_block_device(),
                InstanceType=Ref("InstanceType"),
                KeyName=Ref("SshKeyName"),
                UserData=self.generate_user_data(),
                SecurityGroups=[Ref("DefaultSG"), Ref(CLUSTER_SG_NAME)]))
        t.add_resource(
            autoscaling.AutoScalingGroup(
                "EmpireControllerAutoscalingGroup",
                AvailabilityZones=Ref("AvailabilityZones"),
                LaunchConfigurationName=Ref("EmpireControllerLaunchConfig"),
                MinSize=Ref("MinHosts"),
                MaxSize=Ref("MaxHosts"),
                VPCZoneIdentifier=Ref("PrivateSubnets"),
                Tags=[ASTag("Name", "empire_controller", True)]))



from stacker.context import Context
from stacker.variables import Variable

from stacker_blueprints.kms import Key

from stacker.blueprints.testutil import BlueprintTestCase


class TestKmsKey(BlueprintTestCase):
    def setUp(self):
        self.ctx = Context({'namespace': 'test'})

    def test_kms_key(self):
        blueprint = Key('kms_key_a', self.ctx)
        blueprint.resolve_variables(
            [
                Variable("KeyAlias", "alias/a-test-key"),
                Variable("Properties", {"Description": "a KMS test-key."}),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)

    def test_kms_key_alias_not_in_keyalias(self):
        blueprint = Key('kms_key_b', self.ctx)
        blueprint.resolve_variables(
            [
                Variable("KeyAlias", "b-test-key"),
                Variable("Properties", {"Description": "b KMS test-key."}),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)
    
    def test_kms_key_without_properties(self):
        blueprint = Key('kms_key_c', self.ctx)
        blueprint.resolve_variables(
            [
                Variable("KeyAlias", "alias/c-test-key"),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)

    def test_kms_key_attributes_is_deprecated(self):
        blueprint = Key('kms_key_attributes_deprecated', self.ctx)
        blueprint.resolve_variables(
            [
                Variable("KeyAlias", "c-test-key"),
                Variable("Attributes", {"Description": "c KMS test-key."}),
            ]
        )
        with self.assertRaises(DeprecationWarning):
            blueprint.create_template()


if __name__ == '__main__':
    unittest.main()



def check_properties(properties, allowed_properties, resource):
    """Checks the list of properties in the properties variable against the
    property list provided by the allowed_properties variable. If any property
    does not match the properties in allowed_properties, a ValueError is
    raised to prevent unexpected behavior when creating resources.

    properties: The config (as dict) provided by the configuration file
    allowed_properties: A list of strings representing the available params
        for a resource.
    resource: A string naming the resource in question for the error
        message.
    """
    for key in properties.keys():
        if key not in allowed_properties:
            raise ValueError(
                "%s is not a valid property of %s" % (key, resource)
            )



from stacker.blueprints.base import Blueprint

from troposphere import (
    GetAtt,
    Output,
    Ref,
    iam,
)

from awacs.aws import Policy
from awacs.helpers.trust import (
    get_default_assumerole_policy,
    get_lambda_assumerole_policy
)


class Roles(Blueprint):
    VARIABLES = {
        "Ec2Roles": {
            "type": list,
            "description": "names of ec2 roles to create",
            "default": [],
        },
        "LambdaRoles": {
            "type": list,
            "description": "names of lambda roles to create",
            "default": [],
        },
    }

    def __init__(self, *args, **kwargs):
        super(Roles, self).__init__(*args, **kwargs)
        self.roles = []
        self.policies = []

    def create_role(self, name, assumerole_policy):
        t = self.template

        role = t.add_resource(
            iam.Role(
                name,
                AssumeRolePolicyDocument=assumerole_policy,
            )
        )

        t.add_output(
            Output(name + "RoleName", Value=Ref(role))
        )
        t.add_output(
            Output(name + "RoleArn", Value=GetAtt(role.title, "Arn"))
        )

        self.roles.append(role)
        return role

    def create_ec2_role(self, name):
        return self.create_role(name, get_default_assumerole_policy())

    def create_lambda_role(self, name):
        return self.create_role(name, get_lambda_assumerole_policy())

    def generate_policy_statements(self):
        """Should be overridden on a subclass to create policy statements.

        By subclassing this blueprint, and overriding this method to generate
        a list of :class:`awacs.aws.Statement` types, a
        :class:`troposphere.iam.PolicyType` will be created and attached to
        the roles specified here.

        If not specified, no Policy will be created.
        """

        return []

    def create_policy(self, name):
        statements = self.generate_policy_statements()
        if not statements:
            return

        t = self.template
        policy_prefix = self.context.get_fqn(self.name)

        policy = t.add_resource(
            iam.PolicyType(
                "{}Policy".format(name),
                PolicyName="{}-{}-policy".format(policy_prefix, name),
                PolicyDocument=Policy(
                    Statement=statements,
                ),
                Roles=[Ref(role) for role in self.roles],
            )
        )

        t.add_output(
            Output(name + "PolicyName", Value=Ref(policy))
        )
        self.policies.append(policy)

    def create_template(self):
        variables = self.get_variables()

        for role in variables['Ec2Roles']:
            self.create_ec2_role(role)

        for role in variables['LambdaRoles']:
            self.create_lambda_role(role)

        self.create_policy()



from troposphere import (
    Ref, ec2, Output, GetAtt, Not, Equals, Condition, And, Join
)
from troposphere.rds import DBInstance, DBSubnetGroup
from troposphere.route53 import RecordSetType

from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import (
    CFNNumber,
    CFNString,
    EC2SubnetIdList,
    EC2VPCId,
)

RDS_INSTANCE_NAME = "PostgresRDS%s"
RDS_SUBNET_GROUP = "%sSubnetGroup"
RDS_SG_NAME = "RdsSG%s"


class PostgresRDS(Blueprint):
    VARIABLES = {
        'VpcId': {'type': EC2VPCId, 'description': 'Vpc Id'},
        'PrivateSubnets': {'type': EC2SubnetIdList,
                           'description': 'Subnets to deploy private '
                                          'instances in.'},
        'InstanceType': {'type': CFNString,
                         'description': 'AWS RDS Instance Type',
                         'default': 'db.m3.large'},
        'AllocatedStorage': {'type': CFNNumber,
                             'description': 'Space, in GB, to allocate to RDS '
                                            'instance.',
                             'default': '10'},
        'MasterUser': {'type': CFNString,
                       'description': 'Name of the master user in the db.',
                       'default': 'dbuser'},
        'MasterUserPassword': {'type': CFNString,
                               'description': 'Master user password.'},
        'PreferredBackupWindow': {
            'type': CFNString,
            'description': 'A (minimum 30 minute) window in HH:MM-HH:MM '
                           'format in UTC for backups. Default: 3am-4am',
            'default': '11:00-12:00'},
        'DBName': {
            'type': CFNString,
            'description': 'Initial db to create in database.'},
        "InternalZoneId": {
            "type": CFNString,
            "default": "",
            "description": "Internal zone Id, if you have one."},
        "InternalZoneName": {
            "type": CFNString,
            "default": "",
            "description": "Internal zone name, if you have one."},
        "InternalHostname": {
            "type": CFNString,
            "default": "",
            "description": "Internal domain name, if you have one."},
    }

    def create_conditions(self):
        self.template.add_condition(
            "HasInternalZone",
            Not(Equals(Ref("InternalZoneId"), "")))
        self.template.add_condition(
            "HasInternalZoneName",
            Not(Equals(Ref("InternalZoneName"), "")))
        self.template.add_condition(
            "HasInternalHostname",
            Not(Equals(Ref("InternalHostname"), "")))
        self.template.add_condition(
            "CreateInternalHostname",
            And(Condition("HasInternalZone"),
                Condition("HasInternalZoneName"),
                Condition("HasInternalHostname")))

    def create_subnet_group(self):
        t = self.template
        t.add_resource(
            DBSubnetGroup(
                RDS_SUBNET_GROUP % self.name,
                DBSubnetGroupDescription="%s VPC subnet group." % self.name,
                SubnetIds=Ref('PrivateSubnets')))

    def create_security_group(self):
        t = self.template
        sg_name = RDS_SG_NAME % self.name
        sg = t.add_resource(
            ec2.SecurityGroup(
                sg_name,
                GroupDescription='%s RDS security group' % sg_name,
                VpcId=Ref("VpcId")))
        t.add_output(Output("SecurityGroup", Value=Ref(sg)))

    def create_rds(self):
        t = self.template
        db_name = RDS_INSTANCE_NAME % self.name
        t.add_resource(
            DBInstance(
                db_name,
                AllocatedStorage=Ref('AllocatedStorage'),
                AllowMajorVersionUpgrade=False,
                AutoMinorVersionUpgrade=True,
                BackupRetentionPeriod=30,
                DBName=Ref('DBName'),
                DBInstanceClass=Ref('InstanceType'),
                DBSubnetGroupName=Ref(RDS_SUBNET_GROUP % self.name),
                Engine='postgres',
                EngineVersion='9.3.14',
                MasterUsername=Ref('MasterUser'),
                MasterUserPassword=Ref('MasterUserPassword'),
                MultiAZ=True,
                PreferredBackupWindow=Ref('PreferredBackupWindow'),
                VPCSecurityGroups=[Ref(RDS_SG_NAME % self.name), ]))

        endpoint = GetAtt(db_name, 'Endpoint.Address')

        # Setup CNAME to db
        t.add_resource(
            RecordSetType(
                '%sDnsRecord' % db_name,
                # Appends a '.' to the end of the domain
                HostedZoneId=Ref("InternalZoneId"),
                Comment='RDS DB CNAME Record',
                Name=Join(".", [Ref("InternalHostname"),
                          Ref("InternalZoneName")]),
                Type='CNAME',
                TTL='120',
                ResourceRecords=[endpoint],
                Condition="CreateInternalHostname"))
        t.add_output(Output('DBAddress', Value=endpoint))
        t.add_output(
            Output(
                'DBCname',
                Condition="CreateInternalHostname",
                Value=Ref("%sDnsRecord" % db_name)))

    def create_template(self):
        self.create_conditions()
        self.create_subnet_group()
        self.create_security_group()
        self.create_rds()



import re

from troposphere import (
    Ref, ec2, Output, GetAtt, Tags
)
from troposphere.rds import (
    DBInstance, DBSubnetGroup, DBParameterGroup, OptionGroup,
)
from troposphere.route53 import RecordSetType

from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import CFNString

RDS_ENGINES = ["MySQL", "oracle-se1", "oracle-se", "oracle-ee", "sqlserver-ee",
               "sqlserver-se", "sqlserver-ex", "sqlserver-web", "postgres",
               "aurora"]

# Resource name constants
SUBNET_GROUP = "RDSSubnetGroup"
SECURITY_GROUP = "RDSSecurityGroup"
DBINSTANCE = "RDSDBInstance"
DNS_RECORD = "DBInstanceDnsRecord"


def validate_storage_type(value):
    valid_types = ["", "standard", "gp2", "io1"]
    if value not in valid_types:
        raise ValueError("Invalid storage type: %s." % value)
    return value


def validate_db_instance_identifier(value, allow_empty=True):
    if not value and allow_empty:
        # Empty value will pick up default from stackname
        return value
    l = len(value)
    pattern = r"^[a-zA-Z][a-zA-Z0-9-]*$"
    if not (0 < l < 64):
        raise ValueError("Must be between 1 and 63 characters in length.")
    if not re.match(pattern, value):
        raise ValueError("Must match pattern: %s" % pattern)
    return value


def validate_db_engines(value):
    if value not in RDS_ENGINES:
        raise ValueError(
            "Engine must be one of: %s" % (", ".join(RDS_ENGINES))
        )
    return value


def validate_backup_retention_period(value):
    if not (0 <= value <= 35):
        raise ValueError(
            "Backup retention period must be between 0 and 35."
        )
    return value


class BaseRDS(Blueprint):
    """Base Blueprint for all RDS blueprints.

    Should not be used directly. Either use :class:`MasterInstance` or
    :class:`ReadReplica` classes, or a engine specific blueprint like
    :class:`stacker.blueprints.rds.postgres.MasterInstance` or
    :class:`stacker.blueprints.rds.postgres.ReadReplica`.
    """

    VARIABLES = {
        "DatabaseParameters": {
            "type": dict,
            "default": {},
        },
        "VpcId": {
            "type": str,
            "description": "Vpc Id"},
        "Subnets": {
            "type": str,
            "description": "A comma separated list of subnet ids."},
        "InstanceType": {
            "type": str,
            "description": "AWS RDS Instance Type",
            "default": "db.m3.large"},
        "AllowMajorVersionUpgrade": {
            "type": bool,
            "description": "Set to 'true' to allow major version "
                           "upgrades.",
            "default": False,
        },
        "AutoMinorVersionUpgrade": {
            "type": bool,
            "description": "Set to 'true' to allow minor version upgrades "
                           "during maintenance windows.",
            "default": False,
        },
        "DBFamily": {
            "type": str,
            "description": "DBFamily for ParameterGroup.",
        },
        "StorageType": {
            "type": str,
            "description": "Storage type for RDS instance. Defaults to "
                           "standard unless IOPS is set, then it "
                           "defaults to io1",
            "default": "",
            "validator": validate_storage_type,
        },
        "AllocatedStorage": {
            "type": int,
            "description": "Space, in GB, to allocate to RDS instance. If "
                           "IOPS is set below, this must be a minimum of "
                           "100 and must be at least 1/10th the IOPs "
                           "setting.",
            "default": 0
        },
        "IOPS": {
            "type": int,
            "description": "If set, uses provisioned IOPS for the "
                           "database. Note: This must be no more than "
                           "10x of AllocatedStorage. Minimum: 1000",
            "default": 0
        },
        "InternalZoneId": {
            "type": str,
            "default": "",
            "description": "Internal zone Id, if you have one."
        },
        "InternalZoneName": {
            "type": str,
            "default": "",
            "description": "Internal zone name, if you have one."
        },
        "InternalHostname": {
            "type": str,
            "default": "",
            "description": "Internal domain name, if you have one."
        },
        "PreferredMaintenanceWindow": {
            "type": str,
            "description": "A (minimum 30 minute) window in "
                           "DDD:HH:MM-DDD:HH:MM format in UTC for "
                           "backups. Default: Sunday 3am-4am PST",
            "default": "Sun:11:00-Sun:12:00"
        },
        "DBInstanceIdentifier": {
            "type": str,
            "description": "Name of the database instance in RDS.",
            "validator": validate_db_instance_identifier,
            "default": "",
        },
        "DBSnapshotIdentifier": {
            "type": str,
            "description": "The snapshot you want the db restored from.",
            "default": "",
        },
        "ExistingSecurityGroup": {
            "type": str,
            "description": "The ID of an existing security group to put "
                           "the RDS instance in. If not specified, one "
                           "will be created for you.",
            "default": "",
        },
        "Tags": {
            "type": dict,
            "description": "An optional dictionary of tags to put on the "
                           "database instance.",
            "default": {}
        },
    }

    def engine(self):
        return None

    def defined_variables(self):
        variables = super(BaseRDS, self).defined_variables()

        if not self.engine():
            variables['Engine'] = {
                "type": str,
                "description": "Database engine for the RDS Instance.",
                "validator": validate_db_engines,
            }
        else:
            validate_db_engines(self.engine())

        return variables

    def should_create_internal_hostname(self):
        variables = self.get_variables()
        return all(
            [
                variables["InternalZoneId"],
                variables["InternalZoneName"],
                variables["InternalHostname"]
            ]
        )

    def get_storage_type(self):
        variables = self.get_variables()
        return variables["StorageType"] or Ref("AWS::NoValue")

    def get_db_snapshot_identifier(self):
        variables = self.get_variables()
        return variables["DBSnapshotIdentifier"] or Ref("AWS::NoValue")

    def get_tags(self):
        variables = self.get_variables()
        tag_var = variables["Tags"]
        t = {"Name": self.name}
        t.update(tag_var)
        return Tags(**t)

    def create_subnet_group(self):
        t = self.template
        variables = self.get_variables()
        t.add_resource(
            DBSubnetGroup(
                SUBNET_GROUP,
                DBSubnetGroupDescription="%s VPC subnet group." % self.name,
                SubnetIds=variables["Subnets"].split(",")
            )
        )

    def create_security_group(self):
        t = self.template
        variables = self.get_variables()
        self.security_group = variables["ExistingSecurityGroup"]
        if not variables["ExistingSecurityGroup"]:
            sg = t.add_resource(
                ec2.SecurityGroup(
                    SECURITY_GROUP,
                    GroupDescription="%s RDS security group" % self.name,
                    VpcId=variables["VpcId"]
                )
            )
            self.security_group = Ref(sg)
        t.add_output(Output("SecurityGroup", Value=self.security_group))

    def get_db_endpoint(self):
        endpoint = GetAtt(DBINSTANCE, "Endpoint.Address")
        return endpoint

    def create_parameter_group(self):
        t = self.template
        variables = self.get_variables()
        params = variables["DatabaseParameters"]
        t.add_resource(
            DBParameterGroup(
                "ParameterGroup",
                Description=self.name,
                Family=variables["DBFamily"],
                Parameters=params,
            )
        )

    def get_option_configurations(self):
        options = []
        return options

    def create_option_group(self):
        t = self.template
        variables = self.get_variables()
        t.add_resource(
            OptionGroup(
                "OptionGroup",
                EngineName=self.engine() or variables["Engine"],
                MajorEngineVersion=variables["EngineMajorVersion"],
                OptionGroupDescription=self.name,
                OptionConfigurations=self.get_option_configurations(),
            )
        )

    def create_rds(self):
        t = self.template
        variables = self.get_variables()
        db = DBInstance(
            DBINSTANCE,
            StorageType=self.get_storage_type(),
            **self.get_common_attrs())
        # Hack till https://github.com/cloudtools/troposphere/pull/652/
        # is accepted
        if variables["IOPS"]:
            db.Iops = variables["IOPS"]
        t.add_resource(db)

    def create_dns_records(self):
        t = self.template
        variables = self.get_variables()

        # Setup CNAME to db
        if self.should_create_internal_hostname():
            hostname = "%s.%s" % (
                variables["InternalHostname"],
                variables["InternalZoneName"]
            )
            t.add_resource(
                RecordSetType(
                    DNS_RECORD,
                    # Appends a "." to the end of the domain
                    HostedZoneId=variables["InternalZoneId"],
                    Comment="RDS DB CNAME Record",
                    Name=hostname,
                    Type="CNAME",
                    TTL="120",
                    ResourceRecords=[self.get_db_endpoint()],
                )
            )

    def create_db_outputs(self):
        t = self.template
        t.add_output(Output("DBAddress", Value=self.get_db_endpoint()))
        t.add_output(Output("DBInstance", Value=Ref(DBINSTANCE)))
        if self.should_create_internal_hostname():
            t.add_output(
                Output(
                    "DBCname",
                    Value=Ref(DNS_RECORD)))

    def create_template(self):
        variables = self.get_variables()
        if variables.get("DBFamily"):
            self.create_parameter_group()

        if variables.get("EngineMajorVersion"):
            self.create_option_group()

        self.create_subnet_group()
        self.create_security_group()
        self.create_rds()
        self.create_dns_records()
        self.create_db_outputs()


class MasterInstance(BaseRDS):
    """Blueprint for a generic Master RDS Database Instance.

    Subclasses should be created for each RDS engine for better validation of
    things like engine version.
    """

    def defined_variables(self):
        variables = super(MasterInstance, self).defined_variables()
        additional = {
            "BackupRetentionPeriod": {
                "type": int,
                "description": "Number of days to retain database backups.",
                "validator": validate_backup_retention_period,
                "default": 7,
            },
            "MasterUser": {
                "type": str,
                "description": "Name of the master user in the db.",
            },
            "MasterUserPassword": {
                "type": CFNString,
                "no_echo": True,
                "description": "Master user password."
            },
            "PreferredBackupWindow": {
                "type": str,
                "description": "A (minimum 30 minute) window in HH:MM-HH:MM "
                               "format in UTC for backups. Default: 4am-5am "
                               "PST",
                "default": "12:00-13:00"
            },
            "DatabaseName": {
                "type": str,
                "description": "Initial db to create in database."
            },
            "MultiAZ": {
                "type": bool,
                "description": "Set to 'false' to disable MultiAZ support.",
                "default": True
            },
            "KmsKeyid": {
                "type": str,
                "description": "Requires that StorageEncrypted is true. "
                               "Should be an ARN to the KMS key that should "
                               "be used to encrypt the storage.",
                "default": "",
            },
            "EngineMajorVersion": {
                "type": str,
                "description": "Major Version for the engine. Basically the "
                               "first two parts of the EngineVersion you "
                               "choose."
            },
            "EngineVersion": {
                "type": str,
                "description": "Database engine version for the RDS Instance.",
            },
            "StorageEncrypted": {
                "type": bool,
                "description": "Set to 'false' to disable encrypted storage.",
                "default": True,
            },

        }
        variables.update(additional)
        return variables

    def get_common_attrs(self):
        variables = self.get_variables()
        return {
            "AllocatedStorage": variables["AllocatedStorage"],
            "AllowMajorVersionUpgrade": variables["AllowMajorVersionUpgrade"],
            "AutoMinorVersionUpgrade": variables["AutoMinorVersionUpgrade"],
            "BackupRetentionPeriod": variables["BackupRetentionPeriod"],
            "DBName": variables["DatabaseName"],
            "DBInstanceClass": variables["InstanceType"],
            "DBInstanceIdentifier": (
                variables["DBInstanceIdentifier"]
                or validate_db_instance_identifier(
                    self.context.get_fqn(self.name),
                    allow_empty=False
                )
            ),
            "DBSnapshotIdentifier": self.get_db_snapshot_identifier(),
            "DBParameterGroupName": Ref("ParameterGroup"),
            "DBSubnetGroupName": Ref(SUBNET_GROUP),
            "Engine": self.engine() or variables["Engine"],
            "EngineVersion": variables["EngineVersion"],
            # NoValue for now
            "LicenseModel": Ref("AWS::NoValue"),
            "MasterUsername": variables["MasterUser"],
            "MasterUserPassword": Ref("MasterUserPassword"),
            "MultiAZ": variables["MultiAZ"],
            "OptionGroupName": Ref("OptionGroup"),
            "PreferredBackupWindow": variables["PreferredBackupWindow"],
            "PreferredMaintenanceWindow":
                variables["PreferredMaintenanceWindow"],
            "StorageEncrypted": variables["StorageEncrypted"],
            "VPCSecurityGroups": [self.security_group, ],
            "Tags": self.get_tags(),
        }


class ReadReplica(BaseRDS):
    """Blueprint for a Read replica RDS Database Instance."""

    def defined_variables(self):
        variables = super(ReadReplica, self).defined_variables()
        additional = {
            "MasterDatabaseId": {
                "type": str,
                "description": "ID of the master database to create a read "
                               "replica of."
            },
            "EngineMajorVersion": {
                "type": str,
                "description": "Major Version for the engine. Basically the "
                               "first two parts of the EngineVersion you "
                               "choose."
            },
            "EngineVersion": {
                "type": str,
                "description": "Database engine version for the RDS Instance.",
            },
            "StorageEncrypted": {
                "type": bool,
                "description": "Set to 'false' to disable encrypted storage.",
                "default": True,
            },
        }
        variables.update(additional)
        return variables

    def get_common_attrs(self):
        variables = self.get_variables()

        return {
            "SourceDBInstanceIdentifier": variables["MasterDatabaseId"],
            "AllocatedStorage": variables["AllocatedStorage"],
            "AllowMajorVersionUpgrade": variables["AllowMajorVersionUpgrade"],
            "AutoMinorVersionUpgrade": variables["AutoMinorVersionUpgrade"],
            "DBInstanceClass": variables["InstanceType"],
            "DBInstanceIdentifier": (
                variables["DBInstanceIdentifier"]
                or validate_db_instance_identifier(
                    self.context.get_fqn(self.name),
                    allow_empty=False
                )
            ),
            "DBParameterGroupName": Ref("ParameterGroup"),
            "Engine": self.engine() or variables["Engine"],
            "EngineVersion": variables["EngineVersion"],
            "OptionGroupName": Ref("OptionGroup"),
            "PreferredMaintenanceWindow":
                variables["PreferredMaintenanceWindow"],
            "VPCSecurityGroups": [self.security_group, ],
            "Tags": self.get_tags(),
        }


class ClusterInstance(BaseRDS):
    """Blueprint for an DBCluster Instance."""

    def defined_variables(self):
        variables = super(ClusterInstance, self).defined_variables()
        variables["DBClusterIdentifier"] = {
            "type": str,
            "description": "The database cluster id to join this instance to."
        }
        return variables

    def create_subnet_group(self):
        return

    def get_common_attrs(self):
        variables = self.get_variables()

        return {
            "DBClusterIdentifier": variables["DBClusterIdentifier"],
            "AllowMajorVersionUpgrade": variables["AllowMajorVersionUpgrade"],
            "AutoMinorVersionUpgrade": variables["AutoMinorVersionUpgrade"],
            "DBInstanceClass": variables["InstanceType"],
            "DBInstanceIdentifier": (
                variables["DBInstanceIdentifier"]
                or self.context.get_fqn(self.name)
            ),
            "DBSnapshotIdentifier": self.get_db_snapshot_identifier(),
            "DBParameterGroupName": Ref("ParameterGroup"),
            "Engine": self.engine() or variables["Engine"],
            "LicenseModel": Ref("AWS::NoValue"),
            "Tags": self.get_tags(),
        }



from .base import MasterInstance, ReadReplica


class MySQLMixin(object):
    def engine(self):
        return "MySQL"


class MasterInstance(MySQLMixin, MasterInstance):
    pass


class ReadReplica(MySQLMixin, ReadReplica):
    pass



from stacker.context import Context
from stacker.variables import Variable

from stacker_blueprints.route53 import (
  DNSRecords,
  get_record_set_md5,
)

from stacker.blueprints.testutil import BlueprintTestCase

class TestRoute53(BlueprintTestCase):
    def setUp(self):
        self.ctx = Context({'namespace': 'test'})

    def test_create_template_hosted_zone_id(self):
        blueprint = DNSRecords('route53_dnsrecords', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.1"],
                        },
                        {
                            "Name": "host2.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.2"],
                        },
                    ]
                ),
                Variable("HostedZoneId", "fake_zone_id"),
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)

    def test_create_template_hosted_zone_name(self):
        blueprint = DNSRecords('route53_dnsrecords_zone_name', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.1"],
                        },
                        {
                            "Name": "host2.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.2"],
                            "Comment": "This is host2's record. : )",
                        },
                        {
                            "Name": "host3.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.3"],
                            "Comment": "This record is present but disabled.",
                            "Enabled": False,
                        },
                    ]
                ),
                Variable("HostedZoneName", "testdomain.com"),
                Variable("Comment", "test-testdomain-com"),
            ]
        )
        record_sets = blueprint.create_template()
        self.assertEqual(2, len(record_sets))
        self.assertRenderedBlueprint(blueprint)

    def test_cloudfront_alias_adds_hosted_zone_id(self):
        blueprint = DNSRecords('route53_cf_alias_hosted_zone_id', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "AliasTarget": {
                                "DNSName": "d123456789f.cloudfront.net.",
                            },
                        },
                    ]
                ),
                Variable("HostedZoneId", "fake_zone_id"),
            ]
        )
        record_sets = blueprint.create_template()
        self.assertEqual(record_sets[0].AliasTarget.HostedZoneId, "Z2FDTNDATAQYW2")

    def test_elb_alias_proper_hosted_zone_id(self):
        blueprint = DNSRecords('test_route53_elb_alias_hosted_zone_id', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "AliasTarget": {
                                "DNSName": "myelb-1234567890-abcdef.us-east-2.elb.amazonaws.com.",
                            },
                        },
                    ]
                ),
                Variable("HostedZoneId", "fake_zone_id"),
            ]
        )
        record_sets = blueprint.create_template()
        self.assertEqual(record_sets[0].AliasTarget.HostedZoneId, "Z3AADJGX6KTTL2")

    def test_alias_default_hosted_zone_id(self):
        blueprint = DNSRecords('test_route53_alias_default_hosted_zone_id', self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "AliasTarget": {
                                "DNSName": "original-gangster-host.testdomain.com.",
                            },
                        },
                    ]
                ),
                Variable("HostedZoneId", "fake_zone_id"),
            ]
        )
        record_sets = blueprint.create_template()
        self.assertEqual(record_sets[0].AliasTarget.HostedZoneId, "fake_zone_id")

    def test_error_when_specify_both_hosted_zone_id_and_name(self):
        blueprint = DNSRecords('route53_both_hosted_zone_id_and_name_error',
                               self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.1"],
                        },
                    ]
                ),
                Variable("HostedZoneId", "fake_zone_id"),
                Variable("HostedZoneName", "fake_zone_name"),
            ]
        )
        with self.assertRaises(ValueError):
            blueprint.create_template()

    def test_error_when_specify_no_hosted_zone_id_or_name(self):
        blueprint = DNSRecords('route53_missing_hosted_zone_id_or_name_error',
                               self.ctx)
        blueprint.resolve_variables(
            [
                Variable(
                    "RecordSets",
                    [
                        {
                            "Name": "host.testdomain.com.",
                            "Type": "A",
                            "ResourceRecords": ["10.0.0.1"],
                        },
                    ]
                ),
            ]
        )
        with self.assertRaises(ValueError):
            blueprint.create_template()

    def test_get_record_set_md5(self):
        rs_name = "www.example.com"
        self.assertEqual(
            get_record_set_md5(rs_name, "A"),
            get_record_set_md5(rs_name, "A")
        )
        self.assertNotEqual(
            get_record_set_md5(rs_name, "A"),
            get_record_set_md5(rs_name, "MX")
        )

    def test_get_record_set_md5_a_and_cname_same_sum(self):
        rs_name = "www.example.com"
        self.assertEqual(
            get_record_set_md5(rs_name, "A"),
            get_record_set_md5(rs_name, "CNAME")
        )

    def test_get_record_set_md5_caps_in_name_same_sum(self):
        rs_name = "www.example.com"
        self.assertEqual(
            get_record_set_md5(rs_name, "A"),
            get_record_set_md5("Www.Example.Com", "A")
        )


if __name__ == '__main__':
    unittest.main()



from stacker.blueprints.base import Blueprint

from troposphere import (
    GetAtt,
    Join,
    Output,
    Ref,
    iam,
)

from troposphere import awslambda

import awacs.logs
from awacs.aws import Statement, Allow, Policy
from awacs.helpers.trust import get_lambda_assumerole_policy

NoValue = Ref("AWS::NoValue")
Region = Ref("AWS::Region")
AccountId = Ref("AWS::AccountId")


def lambda_basic_execution_statements(log_group):
    """Basic policy statements for lambda functions.

    Gives Lambda permission to create the Cloudwatch Logs LogGroups and
    LogStreams it needs.
    """
    log_group_parts = ["arn:aws:logs:", Region, ":", AccountId,
                       ":log-group:", log_group]
    log_group_arn = Join("", log_group_parts)
    log_stream_wild = Join("", log_group_parts + [":*"])

    return [
        Statement(
            Effect=Allow,
            Resource=[
                log_group_arn,
                log_stream_wild,
            ],
            Action=[
                awacs.logs.CreateLogGroup,
                awacs.logs.CreateLogStream,
                awacs.logs.PutLogEvents
            ]
        )
    ]


class Function(Blueprint):
    VARIABLES = {
        "Code": {
            "type": awslambda.Code,
            "description": "The troposphere.awslambda.Code object "
                           "returned by the aws lambda hook.",
        },
        "DeadLetterArn": {
            "type": str,
            "description": "Dead Letter Queue (DLQ) Arn (SQS, SNS, etc) "
                           "that AWS Lambda (Lambda) sends events to when it "
                           "can't process them.",
            "default": "",
        },
        "Description": {
            "type": str,
            "description": "Description of the function.",
            "default": "",
        },
        "Environment": {
            "type": dict,
            "description": "Key-value pairs that Lambda caches and makes "
                           "available for your Lambda functions.",
            "default": {},
        },
        "Handler": {
            "type": str,
            "description": "The name of the function (within your source "
                           "code) that Lambda calls to start running your "
                           "code.",
            "default": "handler",
        },
        "KmsKeyArn": {
            "type": str,
            "description": "The Amazon Resource Name (ARN) of an AWS Key "
                           "Management Service (AWS KMS) key that Lambda "
                           "uses to encrypt and decrypt environment variable "
                           "values.",
            "default": "",
        },
        "MemorySize": {
            "type": int,
            "description": "The amount of memory, in MB, that is allocated "
                           "to your Lambda function. Default: 128",
            "default": 128,
        },
        "Runtime": {
            "type": str,
            "description": "The runtime environment for the Lambda function "
                           "that you are uploading.",
        },
        "Timeout": {
            "type": int,
            "description": "The function execution time (in seconds) after "
                           "which Lambda terminates the function. Default: 3",
            "default": 3,
        },
        "VpcConfig": {
            "type": dict,
            "description": "If the Lambda function requires access to "
                           "resources in a VPC, specify a VPC configuration "
                           "that Lambda uses to set up an elastic network "
                           "interface (ENI). Valid keys are: "
                           "SecurityGroupIds (a list of Ids), and SubnetIds "
                           "(a list of Ids)",
            "default": {},
        }
    }

    def dead_letter_config(self):
        arn = self.get_variables()["DeadLetterArn"]
        dlc = NoValue
        if arn:
            dlc = awslambda.DeadLetterConfig(TargetArn=arn)
        return dlc

    def environment(self):
        environment = self.get_variables()["Environment"]
        env = NoValue
        if environment:
            env = awslambda.Environment(Variables=environment)
        return env

    def vpc_config(self):
        vpc_config = self.get_variables()["VpcConfig"]
        config = NoValue
        if vpc_config:
            config = awslambda.VPCConfig(**vpc_config)
        return config

    def generate_policy_statements(self):
        """Generates the policy statements for the role used by the function.

        You should override this, and extend the list that it returns with
        any additional iam policy statements you wish to give the role.

        Returns:
            list: A list of :class:`awacs.aws.Statement` objects.
        """
        log_group = Join("/", ["/aws/lambda", Ref(self.function)])
        return lambda_basic_execution_statements(log_group)

    def create_role(self):
        t = self.template

        self.role = t.add_resource(
            iam.Role(
                "Role",
                AssumeRolePolicyDocument=get_lambda_assumerole_policy(),
            )
        )

        t.add_output(
            Output("RoleName", Value=Ref(self.role))
        )
        t.add_output(
            Output("RoleArn", Value=GetAtt(self.role.title, "Arn"))
        )

    def create_function(self):
        t = self.template
        variables = self.get_variables()

        self.function = t.add_resource(
            awslambda.Function(
                "Function",
                Code=variables["Code"],
                DeadLetterConfig=self.dead_letter_config(),
                Description=variables["Description"] or NoValue,
                Environment=self.environment(),
                Handler=variables["Handler"],
                KmsKeyArn=variables["KmsKeyArn"] or NoValue,
                MemorySize=variables["MemorySize"],
                Role=GetAtt(self.role.title, "Arn"),
                Runtime=variables["Runtime"],
                Timeout=variables["Timeout"],
                VpcConfig=self.vpc_config(),
            )
        )

        t.add_output(
            Output("FunctionName", Value=Ref(self.function))
        )
        t.add_output(
            Output("FunctionArn", Value=GetAtt(self.function.title, "Arn"))
        )

    def create_policy(self):
        t = self.template
        policy_prefix = self.context.get_fqn(self.name)

        self.policy = t.add_resource(
            iam.PolicyType(
                "Policy",
                PolicyName="%s-policy" % policy_prefix,
                PolicyDocument=Policy(
                    Statement=self.generate_policy_statements()
                ),
                Roles=[Ref(self.role)],
            )
        )

        t.add_output(
            Output("PolicyName", Value=Ref(self.policy))
        )

    def create_template(self):
        self.create_role()
        self.create_function()
        self.create_policy()



""" The 'base' cloudformation stack that builds out the virtual datacenter.

This includes the VPC, it's subnets, availability zones, etc.
"""

from troposphere import (
    Ref, Output, Join, FindInMap, Select, GetAZs, Tags,
    GetAtt
)
from troposphere import ec2
from troposphere.route53 import HostedZone, HostedZoneVPCs

from stacker.blueprints.base import Blueprint

NAT_INSTANCE_NAME = 'NatInstance%s'
NAT_GATEWAY_NAME = 'NatGateway%s'
GATEWAY = 'InternetGateway'
GW_ATTACH = 'GatewayAttach'
VPC_NAME = "VPC"
VPC_ID = Ref(VPC_NAME)
DEFAULT_SG = "DefaultSG"
NAT_SG = "NATSG"

NOVALUE = Ref("AWS::NoValue")


class VPC(Blueprint):
    VARIABLES = {
        "AZCount": {
            "type": int,
            "default": 2,
        },
        "PrivateSubnets": {
            "type": list,
            "description": "List of subnets to use for non-public hosts. "
                           "NOTE: Must have as many subnets as AZCount"},
        "PublicSubnets": {
            "type": list,
            "description": "List of subnets to use for public hosts. NOTE: "
                           "Must have as many subnets as AZCount"},
        "BaseDomain": {
            "type": str,
            "default": "",
            "description": "Base domain for the stack."},
        "InternalDomain": {
            "type": str,
            "default": "",
            "description": "Internal domain name, if you have one."},
        "CidrBlock": {
            "type": str,
            "description": "Base CIDR block for subnets.",
            "default": "10.128.0.0/16"},
        "UseNatGateway": {
            "type": bool,
            "description": "If set to false, will configure NAT Instances"
                           "instead of NAT gateways.",
            "default": True},
        "ImageName": {
            "type": str,
            "description": "The image name to use from the AMIMap (usually "
                           "found in the config file.)",
            "default": "NAT"},
        "InstanceType": {
            "type": str,
            "description": "If using NAT Instances, the instance type to use.",
            "default": "m3.medium"},
        "SshKeyName": {
            "type": str,
            "description": "If using NAT Instances, the SSH key to install "
                           "on those instances.",
            "default": ""},
    }

    def create_vpc(self):
        t = self.template
        t.add_resource(ec2.VPC(
            VPC_NAME,
            CidrBlock=self.get_variables()["CidrBlock"], EnableDnsSupport=True,
            EnableDnsHostnames=True))

        # Just about everything needs this, so storing it on the object
        t.add_output(Output("VpcId", Value=VPC_ID))

    def create_internal_zone(self):
        t = self.template
        variables = self.get_variables()

        if variables["InternalDomain"]:
            t.add_resource(
                HostedZone(
                    "InternalZone",
                    Name=variables["InternalDomain"],
                    VPCs=[HostedZoneVPCs(
                        VPCId=VPC_ID,
                        VPCRegion=Ref("AWS::Region"))]
                )
            )
            t.add_output(
                Output(
                    "InternalZoneId",
                    Value=Ref("InternalZone"),
                )
            )
            t.add_output(
                Output(
                    "InternalZoneName",
                    Value=variables["InternalDomain"],
                )
            )

    def create_default_security_group(self):
        t = self.template
        t.add_resource(
            ec2.SecurityGroup(
                DEFAULT_SG,
                VpcId=VPC_ID,
                GroupDescription='Default Security Group'
            )
        )
        t.add_output(
            Output(
                'DefaultSG',
                Value=Ref(DEFAULT_SG)
            )
        )

    def has_hosted_zones(self):
        variables = self.get_variables()
        return any([
            variables["BaseDomain"],
            variables["InternalDomain"]
        ])

    def create_dhcp_options(self):
        t = self.template
        variables = self.get_variables()
        domain_name = Join(
            " ",
            [
                variables["InternalDomain"],
                variables["BaseDomain"]
            ]
        )
        if self.has_hosted_zones():
            dhcp_options = t.add_resource(
                ec2.DHCPOptions(
                    'DHCPOptions',
                    DomainName=domain_name,
                    DomainNameServers=['AmazonProvidedDNS', ],
                )
            )
            t.add_resource(
                ec2.VPCDHCPOptionsAssociation(
                    'DHCPAssociation',
                    VpcId=VPC_ID,
                    DhcpOptionsId=Ref(dhcp_options),
                )
            )
        else:
            dhcp_options = t.add_resource(
                ec2.DHCPOptions(
                    'DHCPOptions',
                    DomainNameServers=['AmazonProvidedDNS', ],
                )
            )
            t.add_resource(
                ec2.VPCDHCPOptionsAssociation(
                    'DHCPAssociation',
                    VpcId=VPC_ID,
                    DhcpOptionsId=Ref(dhcp_options),
                )
            )

    def create_gateway(self):
        t = self.template
        t.add_resource(ec2.InternetGateway(GATEWAY))
        t.add_resource(
            ec2.VPCGatewayAttachment(
                GW_ATTACH,
                VpcId=VPC_ID,
                InternetGatewayId=Ref(GATEWAY)
            )
        )

    def create_network(self):
        t = self.template
        variables = self.get_variables()
        self.create_gateway()
        t.add_resource(ec2.NetworkAcl('DefaultACL',
                                      VpcId=VPC_ID))

        self.create_nat_security_groups()
        subnets = {'public': [], 'private': []}
        net_types = subnets.keys()
        zones = []
        for i in range(variables["AZCount"]):
            az = Select(i, GetAZs(""))
            zones.append(az)
            name_suffix = i
            for net_type in net_types:
                name_prefix = net_type.capitalize()
                subnet_name = "%sSubnet%s" % (name_prefix, name_suffix)
                subnets[net_type].append(subnet_name)
                t.add_resource(
                    ec2.Subnet(
                        subnet_name,
                        AvailabilityZone=az,
                        VpcId=VPC_ID,
                        DependsOn=GW_ATTACH,
                        CidrBlock=variables.get("%sSubnets" % name_prefix)[i],
                        Tags=Tags(type=net_type)
                    )
                )

                route_table_name = "%sRouteTable%s" % (name_prefix,
                                                       name_suffix)
                t.add_resource(
                    ec2.RouteTable(
                        route_table_name,
                        VpcId=VPC_ID,
                        Tags=[ec2.Tag('type', net_type)]
                    )
                )
                t.add_resource(
                    ec2.SubnetRouteTableAssociation(
                        "%sRouteTableAssociation%s" % (name_prefix,
                                                       name_suffix),
                        SubnetId=Ref(subnet_name),
                        RouteTableId=Ref(route_table_name)
                    )
                )

                route_name = '%sRoute%s' % (name_prefix, name_suffix)
                if net_type == 'public':
                    # the public subnets are where the NAT instances live,
                    # so their default route needs to go to the AWS
                    # Internet Gateway
                    t.add_resource(
                        ec2.Route(
                            route_name,
                            RouteTableId=Ref(route_table_name),
                            DestinationCidrBlock="0.0.0.0/0",
                            GatewayId=Ref(GATEWAY)
                        )
                    )
                    self.create_nat_instance(i, subnet_name)
                else:
                    # Private subnets are where actual instances will live
                    # so their gateway needs to be through the nat instances
                    if variables["UseNatGateway"]:
                        instance_id = NOVALUE
                        nat_gateway_id = Ref(NAT_GATEWAY_NAME % name_suffix)
                    else:
                        instance_id = Ref(NAT_INSTANCE_NAME % name_suffix)
                        nat_gateway_id = NOVALUE

                    t.add_resource(
                        ec2.Route(
                            route_name,
                            RouteTableId=Ref(route_table_name),
                            DestinationCidrBlock='0.0.0.0/0',
                            InstanceId=instance_id,
                            NatGatewayId=nat_gateway_id,
                        )
                    )

        for net_type in net_types:
            t.add_output(
                Output(
                    "%sSubnets" % net_type.capitalize(),
                    Value=Join(
                        ",",
                        [Ref(sn) for sn in subnets[net_type]]
                    )
                )
            )

            for i, sn in enumerate(subnets[net_type]):
                t.add_output(
                    Output(
                        "%sSubnet%d" % (net_type.capitalize(), i),
                        Value=Ref(sn)
                    )
                )

        self.template.add_output(Output(
            "AvailabilityZones",
            Value=Join(",", zones)))

        for i, az in enumerate(zones):
            t.add_output(
                Output(
                    "AvailabilityZone%d" % (i),
                    Value=az
                )
            )

    def create_nat_security_groups(self):
        t = self.template
        variables = self.get_variables()

        # Only create security group if NAT Instances are being used.
        if not variables["UseNatGateway"]:
            nat_private_in_all_rule = ec2.SecurityGroupRule(
                IpProtocol='-1', FromPort='-1', ToPort='-1',
                SourceSecurityGroupId=Ref(DEFAULT_SG)
            )

            nat_public_out_all_rule = ec2.SecurityGroupRule(
                IpProtocol='-1', FromPort='-1', ToPort='-1', CidrIp='0.0.0.0/0'
            )

            return t.add_resource(
                ec2.SecurityGroup(
                    NAT_SG,
                    VpcId=VPC_ID,
                    GroupDescription='NAT Instance Security Group',
                    SecurityGroupIngress=[nat_private_in_all_rule],
                    SecurityGroupEgress=[nat_public_out_all_rule, ]
                )
            )

    def create_nat_instance(self, zone_id, subnet_name):
        t = self.template
        variables = self.get_variables()
        suffix = zone_id
        eip_name = "NATExternalIp%s" % suffix

        if variables["UseNatGateway"]:
            gateway_name = NAT_GATEWAY_NAME % suffix
            t.add_resource(
                ec2.NatGateway(
                    gateway_name,
                    AllocationId=GetAtt(eip_name, 'AllocationId'),
                    SubnetId=Ref(subnet_name),
                )
            )

            t.add_output(
                Output(
                    gateway_name + "Id",
                    Value=Ref(gateway_name)
                )
            )

            # Using NAT Gateways, leave the EIP unattached - it gets allocated
            # to the NAT Gateway in that resource above
            eip_instance_id = Ref("AWS::NoValue")
        else:
            image_id = FindInMap(
                'AmiMap',
                Ref("AWS::Region"),
                Ref("ImageName")
            )
            instance_name = NAT_INSTANCE_NAME % suffix
            t.add_resource(
                ec2.Instance(
                    instance_name,
                    Condition="UseNatInstances",
                    ImageId=image_id,
                    SecurityGroupIds=[Ref(DEFAULT_SG), Ref(NAT_SG)],
                    SubnetId=Ref(subnet_name),
                    InstanceType=variables["InstanceType"],
                    SourceDestCheck=False,
                    KeyName=variables["SshKeyName"],
                    Tags=[ec2.Tag('Name', 'nat-gw%s' % suffix)],
                    DependsOn=GW_ATTACH
                )
            )
            t.add_output(
                Output(
                    instance_name + "PublicHostname",
                    Value=GetAtt(instance_name, "PublicDnsName")
                )
            )
            t.add_output(
                Output(
                    instance_name + "InstanceId",
                    Value=Ref(instance_name)
                )
            )

            # Since we're using NAT instances, go ahead and attach the EIP
            # to the NAT instance
            eip_instance_id = Ref(instance_name)

        t.add_resource(
            ec2.EIP(
                eip_name,
                Domain='vpc',
                InstanceId=eip_instance_id,
                DependsOn=GW_ATTACH
            )
        )

    def create_template(self):
        self.create_vpc()
        self.create_internal_zone()
        self.create_default_security_group()
        self.create_dhcp_options()
        self.create_network()



import logging

from awacs.aws import (
    Allow,
    AWSPrincipal,
    Policy,
    Statement,
)

import awacs.kms

from troposphere import (
    Join,
    Output,
    Ref,
    kms,
)

from stacker.blueprints.base import Blueprint

logger = logging.getLogger(__name__)


def kms_key_root_statements():
    root_arn = Join(":", ["arn:aws:iam:", Ref("AWS::AccountId"), "root"])

    return [
        Statement(
            Sid="Enable IAM User Permissions",
            Effect=Allow,
            Principal=AWSPrincipal(root_arn),
            Action=[
                awacs.kms.Action("*"),
            ],
            Resource=["*"]
        )
    ]


def kms_key_policy():
    """ Creates a key policy for use of a KMS Key.  """

    statements = []
    statements.extend(kms_key_root_statements())

    return Policy(
        Version="2012-10-17",
        Id="root-account-access",
        Statement=statements
    )


class Key(Blueprint):
    VARIABLES = {
        "KeyAlias": {
            "type": str,
            "description": "The alias to give the key.",
            "default": "",
        },
        "Properties": {
            "type": dict,
            "description": "A dictionary of KMS key attributes which should "
                           "match the attributes for AWS::KMS::Key "
                           "Cloudformation resource. Note: You should "
                           "not supply a `KeyPolicy` attribute.",
            "default": {},
        },
        "Attributes": {
            "type": dict,
            "description": "Deprecated. Use Properties instead.",
            "default": {},
        }
    }

    def generate_key_policy(self):
        return kms_key_policy()

    def create_template(self):
        t = self.template
        variables = self.get_variables()

        key_policy = self.generate_key_policy()
        props = variables["Properties"]

        if variables["Attributes"]:
            raise DeprecationWarning(
                    "Attributes was deprecated, use Properties instead.")

        if "KeyPolicy" in props:
            logger.warning("KeyPolicy provided, but not used. To write "
                           "your own policy you need to subclass this "
                           "blueprint and override `generate_key_policy`.")
        props["KeyPolicy"] = key_policy

        key = t.add_resource(
            kms.Key.from_dict("Key", props)
        )

        key_arn = Join(
            "",
            [
                "arn:aws:kms:",
                Ref("AWS::Region"),
                ":",
                Ref("AWS::AccountId"),
                ":key/",
                Ref(key)
            ]
        )

        t.add_output(Output("KeyArn", Value=key_arn))
        t.add_output(Output("KeyId", Value=Ref(key)))

        key_alias = variables["KeyAlias"]
        if key_alias:
            if not key_alias.startswith("alias/"):
                key_alias = "alias/%s" % key_alias
            alias = t.add_resource(
                kms.Alias(
                    "Alias",
                    AliasName="%s" % key_alias,
                    TargetKeyId=Ref(key)
                )
            )

            t.add_output(Output("KeyAlias", Value=Ref(alias)))



from .base import MasterInstance, ReadReplica


class PostgresMixin(object):
    def engine(self):
        return "postgres"


class MasterInstance(PostgresMixin, MasterInstance):
    pass


class ReadReplica(PostgresMixin, ReadReplica):
    pass



from troposphere import firehose, logs, Output, Ref, GetAtt

from stacker.blueprints.variables.types import CFNString

from .base import BaseDeliveryStream

DELIVERY_STREAM = "DeliveryStream"
REDSHIFT_LOG_STREAM = "RedshiftLogStream"


class DeliveryStream(BaseDeliveryStream):
    def defined_variables(self):
        variables = super(DeliveryStream, self).defined_variables()

        additional = {
            "JDBCURL": {
                "type": str,
                "description": "The URL used to connect to redshift"
            },
            "Username": {
                "type": str,
                "description": "The user for the redshift table"
            },
            "Password": {
                "type": CFNString,
                "description": "The password for the redshift user",
                "no_echo": True,
            },
            "TableName": {
                "type": str,
                "description": "The redshift table"
            },
            "CopyOptions": {
                "type": str,
                "description": "Copy Options used by the redshift copy "
                               "command.",
                "default": "JSON 'auto' ACCEPTINVCHARS BLANKSASNULL "
                           "EMPTYASNULL GZIP STATUPDATE OFF COMPUPDATE OFF",
            },
        }

        variables.update(additional)
        return variables

    def create_log_stream(self):
        t = self.template
        super(DeliveryStream, self).create_log_stream()

        self.redshift_log_stream = t.add_resource(
            logs.LogStream(
                REDSHIFT_LOG_STREAM,
                LogGroupName=Ref(self.log_group),
                DependsOn=self.log_group.title
            )
        )

        t.add_output(
            Output(
                "RedshiftLogStreamName",
                Value=Ref(self.redshift_log_stream)
            )
        )

    def create_delivery_stream(self):
        t = self.template
        variables = self.get_variables()

        s3_dest_config = firehose.S3Configuration(
            **self.s3_destination_config_dict()
        )

        redshift_config = firehose.RedshiftDestinationConfiguration(
            RoleARN=GetAtt(self.role, "Arn"),
            ClusterJDBCURL=variables['JDBCURL'],
            CopyCommand=firehose.CopyCommand(
                CopyOptions=variables["CopyOptions"],
                DataTableName=variables['TableName']
            ),
            Username=variables['Username'],
            Password=variables['Password'].ref,
            S3Configuration=s3_dest_config,
            CloudWatchLoggingOptions=self.cloudwatch_logging_options(
                self.log_group,
                self.redshift_log_stream
            )
        )

        self.delivery_stream = t.add_resource(
            firehose.DeliveryStream(
                DELIVERY_STREAM,
                RedshiftDestinationConfiguration=redshift_config
            )
        )



"""AWS Elasticsearch Service.

Blueprint to configure AWS Elasticsearch service.

Example::

    - name: elasticsearch
      class_path: stacker_blueprints.elasticsearch.Domain
      variables:
        Roles:
          - ${empireMinion::IAMRole}
        InternalZoneId: ${vpc::InternalZoneId}
        InternalZoneName: ${vpc::InternalZoneName}
        InternalHostName: es

"""
import awacs.es
from awacs.aws import (
    Allow,
    Condition,
    IpAddress,
    Policy,
    Principal,
    Everybody,
    SourceIp,
    Statement,
)
from stacker.blueprints.base import Blueprint
from troposphere import (
    elasticsearch,
    iam,
    route53,
    GetAtt,
    Join,
    Output,
    Ref,
)

ES_DOMAIN = "ESDomain"
DNS_RECORD = "ESDomainDNSRecord"
POLICY_NAME = "ESDomainAccessPolicy"


class Domain(Blueprint):

    VARIABLES = {
        "Roles": {
            "type": list,
            "description": (
                "List of roles that should have access to the ES domain.")},
        "InternalZoneId": {
            "type": str,
            "default": "",
            "description": "Internal zone id, if you have one."},
        "InternalZoneName": {
            "type": str,
            "default": "",
            "description": "Internal zone name, if you have one."},
        "InternalHostName": {
            "type": str,
            "default": "",
            "description": "Internal domain name, if you have one."},
        "AdvancedOptions": {
            "type": dict,
            "default": {},
            "description": (
                "Additional options to specify for the Amazon ES domain"
            )},
        "DomainName": {
            "type": str,
            "default": "",
            "description": "A name for the Amazon ES domain."},
        "EBSOptions": {
            "type": dict,
            "default": {},
            "description": (
                "The configurations of Amazon Elastic Block Store (Amazon "
                "EBS) volumes that are attached to data nodes in the Amazon "
                "ES domain"
            )},
        "ElasticsearchClusterConfig": {
            "type": dict,
            "default": {},
            "description": (
                "The cluster configuration for the Amazon ES domain."
            )},
        "ElasticsearchVersion": {
            "type": str,
            "default": "2.3",
            "description": "The version of Elasticsearch to use."},
        "SnapshotOptions": {
            "type": dict,
            "default": {},
            "description": (
                "The automated snapshot configuration for the Amazon ES "
                "domain indices."
            )},
        "Tags": {
            "type": list,
            "default": [],
            "description": (
                "An arbitrary set of tags (key-value pairs) to associate with "
                "the Amazon ES domain."
            )},
        "TrustedNetworks": {
            "type": list,
            "description": (
                "List of CIDR blocks allowed to connect to the ES cluster"
            ),
            "default": []},
    }

    def get_allowed_actions(self):
        return [
            awacs.es.Action("ESHttpGet"),
            awacs.es.Action("ESHttpHead"),
            awacs.es.Action("ESHttpPost"),
            awacs.es.Action("ESHttpDelete")]

    def create_dns_record(self):
        t = self.template
        variables = self.get_variables()
        should_create_dns = all([
            variables["InternalZoneId"],
            variables["InternalZoneName"],
            variables["InternalHostName"],
        ])
        if should_create_dns:
            t.add_resource(
                route53.RecordSetType(
                    DNS_RECORD,
                    HostedZoneId=variables["InternalZoneId"],
                    Comment="ES Domain CNAME Record",
                    Name="{}.{}".format(variables["InternalHostName"],
                                        variables["InternalZoneName"]),
                    Type="CNAME",
                    TTL="120",
                    ResourceRecords=[GetAtt(ES_DOMAIN, "DomainEndpoint")],
                ))
            t.add_output(Output("CNAME", Value=Ref(DNS_RECORD)))

    def create_domain(self):
        t = self.template
        variables = self.get_variables()
        params = {
            "ElasticsearchVersion": variables["ElasticsearchVersion"],
        }

        policy = self.get_access_policy()
        if policy:
            params["AccessPolicies"] = policy

        # Add any optional keys to the params dict. ES didn't have great
        # support for passing empty values for these keys when this was
        # created.
        optional_keys = ["AdvancedOptions", "DomainName", "EBSOptions",
                         "SnapshotOptions", "Tags"]

        for key in optional_keys:
            optional = variables[key]
            if optional:
                params[key] = optional

        domain = elasticsearch.Domain.from_dict(ES_DOMAIN, params)
        t.add_resource(domain)
        t.add_output(Output("DomainArn", Value=GetAtt(ES_DOMAIN, "DomainArn")))
        t.add_output(Output("DomainEndpoint", Value=GetAtt(ES_DOMAIN,
                                                           "DomainEndpoint")))

    def create_roles_policy(self):
        t = self.template
        variables = self.get_variables()
        statements = [
            Statement(
                Effect=Allow,
                Action=self.get_allowed_actions(),
                Resource=[Join("/", [GetAtt(ES_DOMAIN, "DomainArn"), "*"])])]
        t.add_resource(
            iam.PolicyType(
                POLICY_NAME,
                PolicyName=POLICY_NAME,
                PolicyDocument=Policy(Statement=statements),
                Roles=variables["Roles"]))

    def get_access_policy(self):
        policy = None
        variables = self.get_variables()

        statements = []
        for trusted_network in variables["TrustedNetworks"]:
            condition = Condition(IpAddress({SourceIp: trusted_network}))
            statements.append(
                Statement(
                    Effect=Allow,
                    Action=self.get_allowed_actions(),
                    Condition=condition,
                    Principal=Principal(Everybody)))

        if statements:
            policy = Policy(Statement=statements)
        return policy

    def create_template(self):
        self.create_domain()
        self.create_dns_record()
        self.create_roles_policy()



from troposphere import firehose

from .base import BaseDeliveryStream

DELIVERY_STREAM = "DeliveryStream"


class DeliveryStream(BaseDeliveryStream):
    def create_delivery_stream(self):
        t = self.template

        s3_dest_config = firehose.S3DestinationConfiguration(
            **self.s3_destination_config_dict()
        )

        self.delivery_stream = t.add_resource(
            firehose.DeliveryStream(
                DELIVERY_STREAM,
                S3DestinationConfiguration=s3_dest_config
            )
        )



from stacker.blueprints.base import Blueprint
from troposphere import (
    Output,
    Ref,
    GetAtt,
    s3,
    iam,
)

from .policies import (
    read_only_s3_bucket_policy,
    read_write_s3_bucket_policy,
    s3_arn
)


class Buckets(Blueprint):
    VARIABLES = {
        "Buckets": {
            "type": dict,
            "description": "A dictionary of buckets to create. The key "
                           "being the CFN logical resource name, the "
                           "value being a dictionary of attributes for "
                           "the troposphere s3.Bucket type.",
            "default": {}
        },
        "ReadWriteRoles": {
            "type": list,
            "description": "A list of roles that should have read/write "
                           "access to the buckets created.",
            "default": []
        },
        "ReadRoles": {
            "type": list,
            "description": "A list of roles that should have read-only "
                           "access to the buckets created.",
            "default": []
        },

    }

    def create_template(self):
        t = self.template
        variables = self.get_variables()

        policy_prefix = self.context.get_fqn(self.name)

        bucket_ids = []

        for title, attrs in variables["Buckets"].items():
            t.add_resource(s3.Bucket.from_dict(title, attrs))
            t.add_output(Output(title + "BucketId", Value=Ref(title)))
            t.add_output(Output(title + "BucketArn", Value=s3_arn(Ref(title))))
            t.add_output(
                Output(
                    title + "BucketDomainName",
                    Value=GetAtt(title, "DomainName")
                )
            )
            if "WebsiteConfiguration" in attrs:
                t.add_output(
                    Output(
                        title + "WebsiteUrl",
                        Value=GetAtt(title, "WebsiteURL")
                    )
                )

            bucket_ids.append(Ref(title))

        read_write_roles = variables["ReadWriteRoles"]
        if read_write_roles:
            t.add_resource(
                iam.PolicyType(
                    "ReadWritePolicy",
                    PolicyName=policy_prefix + "ReadWritePolicy",
                    PolicyDocument=read_write_s3_bucket_policy(
                        bucket_ids
                    ),
                    Roles=read_write_roles,
                )
            )

        read_only_roles = variables["ReadRoles"]
        if read_only_roles:
            t.add_resource(
                iam.PolicyType(
                    "ReadPolicy",
                    PolicyName=policy_prefix + "ReadPolicy",
                    PolicyDocument=read_only_s3_bucket_policy(
                        bucket_ids
                    ),
                    Roles=read_only_roles,
                )
            )



from hashlib import md5

from stacker.blueprints.base import Blueprint

from troposphere import (
    Ref,
    Output,
    GetAtt,
    Join,
    route53,
)

import logging
logger = logging.getLogger(__name__)

# reference: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html  # noqa
CLOUDFRONT_ZONE_ID = "Z2FDTNDATAQYW2"

# reference: http://docs.aws.amazon.com/general/latest/gr/rande.html
ELB_ZONE_IDS = {
    'us-east-2': 'Z3AADJGX6KTTL2',
    'us-east-1': 'Z35SXDOTRQ7X7K',
    'us-west-1': 'Z368ELLRRE2KJ0',
    'us-west-2': 'Z1H1FL5HABSF5',
    'ca-central-1': 'ZQSVJUPU6J1EY',
    'ap-south-1': 'ZP97RAFLXTNZK',
    'ap-northeast-2': 'ZWKZPGTI48KDX',
    'ap-southeast-1': 'Z1LMS91P8CMLE5',
    'ap-southeast-2': 'Z1GM3OXH4ZPM65',
    'ap-northeast-1': 'Z14GRHDCWA56QT',
    'eu-central-1': 'Z215JYRZR1TBD5',
    'eu-west-1': 'Z32O12XQLNTSW2',
    'eu-west-2': 'ZHURV8PSTC4K8',
    'sa-east-1': 'Z2P70J7HTTTPLU',
}

CF_DOMAIN = ".cloudfront.net."
ELB_DOMAIN = ".elb.amazonaws.com."


def get_record_set_md5(rs_name, rs_type):
    """Accept record_set Name and Type. Return MD5 sum of these values."""
    rs_name = rs_name.lower()
    rs_type = rs_type.upper()
    # Make A and CNAME records hash to same sum to support updates.
    rs_type = "ACNAME" if rs_type in ["A", "CNAME"] else rs_type
    return md5(rs_name + rs_type).hexdigest()


def add_hosted_zone_id_if_missing(record_set, hosted_zone_id):
    """Add HostedZoneId to Trophosphere record_set object if missing."""
    if not getattr(record_set, "HostedZoneId", None):
        record_set.HostedZoneId = hosted_zone_id
    return record_set


class DNSRecords(Blueprint):

    VARIABLES = {
        "HostedZoneId": {
            "type": str,
            "description": "The id of an existing HostedZone.",
            "default": "",
        },
        "HostedZoneName": {
            "type": str,
            "description": "The name of a HostedZone to create and manage.",
            "default": "",
        },
        "Comment": {
            "type": str,
            "description": "The comment of a stacker managed HostedZone."
                           "Does nothing when HostedZoneId in variables.",
            "default": "",
        },
        "RecordSets": {
            "type": list,
            "description": "A list of dictionaries representing the attributes"
                           "of a troposphere.route53.RecordSetType object."
                           "Also accepts an optional 'Enabled' boolean.",
            "default": []
        },
    }

    def add_hosted_zone_id_for_alias_target_if_missing(self, rs):
        """Add proper hosted zone id to record set alias target if missing."""
        if getattr(rs, "AliasTarget", None):
            if not getattr(rs.AliasTarget, "HostedZoneId", None):
                if rs.AliasTarget.DNSName.endswith(CF_DOMAIN):
                    rs.AliasTarget.HostedZoneId = CLOUDFRONT_ZONE_ID
                elif rs.AliasTarget.DNSName.endswith(ELB_DOMAIN):
                    elb_region = rs.AliasTarget.DNSName.split('.')[-5]
                    rs.AliasTarget.HostedZoneId = ELB_ZONE_IDS[elb_region]
                else:
                    rs.AliasTarget.HostedZoneId = self.hosted_zone_id
        return rs

    def create_record_set(self, rs_dict):
        """Accept a record_set dict. Return a Troposphere record_set object."""
        record_set_md5 = get_record_set_md5(rs_dict["Name"], rs_dict["Type"])
        rs = route53.RecordSetType.from_dict(record_set_md5, rs_dict)
        rs = add_hosted_zone_id_if_missing(rs, self.hosted_zone_id)
        rs = self.add_hosted_zone_id_for_alias_target_if_missing(rs)
        return self.template.add_resource(rs)

    def create_record_sets(self, record_set_dicts):
        """Accept list of record_set dicts.
        Return list of record_set objects."""
        record_set_objects = []
        for record_set_dict in record_set_dicts:
            # pop removes the 'Enabled' key and tests if True.
            if record_set_dict.pop('Enabled', True):
                record_set_objects.append(
                    self.create_record_set(record_set_dict)
                )
        return record_set_objects

    def create_template(self):
        variables = self.get_variables()
        hosted_zone_name = variables["HostedZoneName"]
        hosted_zone_id = variables["HostedZoneId"]
        hosted_zone_comment = variables["Comment"]

        if all([hosted_zone_comment, hosted_zone_id]):
            logger.warning(
                "The Comment variable works when HostedZoneName is passed."
                "When HostedZoneId in variables, Comment is ignored."
            )

        if all([hosted_zone_name, hosted_zone_id]):
            raise ValueError("Cannot specify both 'HostedZoneName' and "
                             "'HostedZoneId' variables.")

        if not any([hosted_zone_name, hosted_zone_id]):
            raise ValueError("Please specify either a 'HostedZoneName' or "
                             "'HostedZoneId' variable.")

        if hosted_zone_id:
            self.hosted_zone_id = hosted_zone_id

        else:
            hosted_zone_config = route53.HostedZoneConfiguration(
                "HostedZoneConfiguration",
                Comment=hosted_zone_comment
            )
            self.template.add_resource(
                route53.HostedZone(
                    "HostedZone",
                    Name=hosted_zone_name,
                    HostedZoneConfig=hosted_zone_config
                )
            )
            self.hosted_zone_id = Ref("HostedZone")
            self.nameservers = Join(',', GetAtt("HostedZone", "NameServers"))
            self.template.add_output(
                    Output("NameServers", Value=self.nameservers))

        self.template.add_output(
                Output("HostedZoneId", Value=self.hosted_zone_id))

        # return a list of troposphere record set objects.
        return self.create_record_sets(variables["RecordSets"])



import unittest

from stacker.blueprints.testutil import BlueprintTestCase
from stacker.context import Context
from stacker.variables import Variable

from stacker_blueprints.cloudwatch_logs import SubscriptionFilters

from troposphere import GetAtt, Ref


class TestSubscriptionFilters(BlueprintTestCase):
    def setUp(self):
        self.ctx = Context({'namespace': 'test'})

    def test_create_template(self):
        blueprint = SubscriptionFilters(
            'test_cloudwatch_logs_subscription_filters',
            self.ctx
        )

        blueprint.resolve_variables(
            [
                Variable(
                    "SubscriptionFilters",
                    {
                        "Filter1": {
                            "DestinationArn": GetAtt("KinesisStream1", "Arn"),
                            "FilterPattern": "{$.userIdentity.type = Root}",
                            "LogGroupName": Ref("LogGroup1"),
                        },
                        "Filter2": {
                            "DestinationArn": GetAtt("KinesisStream2", "Arn"),
                            "FilterPattern": "{$.userIdentity.type = Root}",
                            "LogGroupName": Ref("LogGroup2"),
                        },
                    }
                )
            ]
        )
        blueprint.create_template()
        self.assertRenderedBlueprint(blueprint)


if __name__ == '__main__':
    unittest.main()



from . import base


class RedisReplicationGroup(base.BaseReplicationGroup):
    def engine(self):
        return "redis"

    def get_engine_versions(self):
        return ["2.6.13", "2.8.19", "2.8.21", "2.8.22", "2.8.23", "2.8.24",
                "2.8.6", "3.2.4"]

    def get_parameter_group_family(self):
        return ["redis2.6", "redis2.8", "redis3.2"]



from troposphere import (
    Ref, ec2, Output, GetAtt, Join
)

from troposphere.elasticache import (
    ReplicationGroup, ParameterGroup, SubnetGroup
)

from troposphere.route53 import RecordSetType

from stacker.blueprints.base import Blueprint

# Resource name constants
SUBNET_GROUP = "SubnetGroup"
SECURITY_GROUP = "SecurityGroup"
REPLICATION_GROUP = "ReplicationGroup"
DNS_RECORD = "ReplicationGroupDnsRecord"
PARAMETER_GROUP = "ParameterGroup"

NOVALUE = Ref("AWS::NoValue")


class BaseReplicationGroup(Blueprint):
    """Base Blueprint for all Elasticache ReplicationGroup blueprints.

    ReplicationGroups are only currently supported by the redis engine.
    """

    ALLOWED_ENGINES = ["redis"]

    VARIABLES = {
        "ClusterParameters": {
            "type": dict,
            "default": {},
        },
        "VpcId": {
            "type": str,
            "description": "Vpc Id to place the Cluster in"
        },
        "Subnets": {
            "type": str,
            "description": "Comma separated list of subnets to deploy the "
                           "Cluster nodes in."
        },
        "AutomaticFailoverEnabled": {
            "type": bool,
            "description": "Specifies whether a read-only replica will be "
                           "automatically promoted to read/write primary "
                           "if the existing primary fails. If true, "
                           "Multi-AZ is enabled for this replication "
                           "group. If false, Multi-AZ is disabled for "
                           "this replication group. If true, "
                           "NumCacheClusters must be at least 2.",
            "default": True,
        },
        "AutoMinorVersionUpgrade": {
            "type": bool,
            "description": "Set to 'true' to allow minor version upgrades "
                           "during maintenance windows.",
        },
        "CacheNodeType": {
            "type": str,
            "description": "AWS ElastiCache Cache Node Type",
        },
        "EngineVersion": {
            "type": str,
            "description": "Engine version for the Cache Cluster.",
        },
        "NotificationTopicArn": {
            "type": str,
            "description": "ARN of the SNS Topic to publish events to.",
            "default": "",
        },
        "NumCacheClusters": {
            "type": int,
            "description": "The number of cache clusters this replication "
                           "group will initially have. If Multi-AZ "
                           "(ie: the AutomaticFailoverEnabled Parameter) "
                           "is enabled, the value of this parameter must "
                           "be at least 2.",
            "default": 2,
        },
        "Port": {
            "type": int,
            "description": "The port to run the cluster on.",
            "default": 0,
        },
        "PreferredCacheClusterAZs": {
            "type": list,
            "description": "Must match the # of nodes in "
                           "NumCacheClusters.",
            "default": [],
        },
        "PreferredMaintenanceWindow": {
            "type": str,
            "description": "A (minimum 60 minute) window in "
                           "DDD:HH:MM-DDD:HH:MM format in UTC for "
                           "backups. Default: Sunday 3am-4am PST",
            "default": "Sun:11:00-Sun:12:00"
        },
        "SnapshotArns": {
            "type": list,
            "description": "A list of s3 ARNS where redis snapshots are "
                           "stored that will be used to create the "
                           "cluster.",
            "default": [],
        },
        "SnapshotRetentionLimit": {
            "type": int,
            "description": "The number of daily snapshots to retain. Only "
                           "valid for clusters with the redis Engine.",
            "default": 0,
        },
        "SnapshotWindow": {
            "type": str,
            "description": "For Redis cache clusters, daily time range "
                           "(in UTC) during which ElastiCache will begin "
                           "taking a daily snapshot of your node group. "
                           "For example, you can specify 05:00-09:00.",
            "default": ""
        },
        "InternalZoneId": {
            "type": str,
            "description": "Internal zone Id, if you have one.",
            "default": "",
        },
        "InternalZoneName": {
            "type": str,
            "description": "Internal zone name, if you have one.",
            "default": "",
        },
        "InternalHostname": {
            "type": str,
            "description": "Internal domain name, if you have one.",
            "default": "",
        },
    }

    def engine(self):
        return None

    def get_engine_versions(self):
        """Used by engine specific subclasses - returns valid engine versions.

        Should only be overridden if the class variable ENGINE is defined on
        the class.

        Return:
            list: A list of valid engine versions for the given engine.
        """
        return []

    def get_parameter_group_family(self):
        """Used by engine specific subclasses to return parameter group family.

        Should only be overridden if the class variable ENGINE is defined on
        the class.

        Return:
            list: A list of valid parameter group families for the given
                  engine.
        """
        return []

    def defined_variables(self):
        variables = super(BaseReplicationGroup, self).defined_variables()
        variables["ParameterGroupFamily"] = {
            "type": str,
            "description": "The parametergroup family to use, dependent "
                           "on the engine.",
            "allowed_values": self.get_parameter_group_family()
        }
        engine_versions = self.get_engine_versions()
        if engine_versions:
            variables['EngineVersion']['allowed_values'] = engine_versions

        if self.engine() not in self.ALLOWED_ENGINES:
            raise ValueError("ENGINE must be one of: %s" %
                             ", ".join(self.ALLOWED_ENGINES))

        return variables

    def create_parameter_group(self):
        t = self.template
        variables = self.get_variables()
        params = variables["ClusterParameters"]
        t.add_resource(
            ParameterGroup(
                PARAMETER_GROUP,
                Description=self.name,
                CacheParameterGroupFamily=variables["ParameterGroupFamily"],
                Properties=params,
            )
        )

    def create_subnet_group(self):
        t = self.template
        t.add_resource(
            SubnetGroup(
                SUBNET_GROUP,
                Description="%s subnet group." % self.name,
                SubnetIds=self.get_variables()["Subnets"].split(',')))

    def create_security_group(self):
        t = self.template
        sg = t.add_resource(
            ec2.SecurityGroup(
                SECURITY_GROUP,
                GroupDescription="%s security group" % self.name,
                VpcId=self.get_variables()["VpcId"]))
        t.add_output(Output("SecurityGroup", Value=Ref(sg)))

    def create_replication_group(self):
        t = self.template
        variables = self.get_variables()
        availability_zones = variables["PreferredCacheClusterAZs"] or NOVALUE
        notification_topic_arn = variables["NotificationTopicArn"] or \
            NOVALUE
        port = variables["Port"] or NOVALUE
        snapshot_arns = variables["SnapshotArns"] or NOVALUE
        snapshot_retention_limit = variables["SnapshotRetentionLimit"] or \
            NOVALUE
        snapshot_window = variables["SnapshotWindow"] or NOVALUE
        maintenance_window = variables["PreferredMaintenanceWindow"] or \
            NOVALUE

        t.add_resource(
            ReplicationGroup(
                REPLICATION_GROUP,
                AutomaticFailoverEnabled=variables["AutomaticFailoverEnabled"],
                AutoMinorVersionUpgrade=variables["AutoMinorVersionUpgrade"],
                CacheNodeType=variables["CacheNodeType"],
                CacheParameterGroupName=Ref(PARAMETER_GROUP),
                CacheSubnetGroupName=Ref(SUBNET_GROUP),
                NumCacheClusters=variables["NumCacheClusters"],
                Engine=self.engine(),
                EngineVersion=variables["EngineVersion"],
                NotificationTopicArn=notification_topic_arn,
                Port=port,
                PreferredCacheClusterAZs=availability_zones,
                PreferredMaintenanceWindow=maintenance_window,
                ReplicationGroupDescription=self.name,
                SecurityGroupIds=[Ref(SECURITY_GROUP), ],
                SnapshotArns=snapshot_arns,
                SnapshotRetentionLimit=snapshot_retention_limit,
                SnapshotWindow=snapshot_window,
            )
        )

    def get_primary_address(self):
        return GetAtt(REPLICATION_GROUP, "PrimaryEndPoint.Address")

    def get_secondary_addresses(self):
        return GetAtt(REPLICATION_GROUP, "ReadEndPoint.Addresses.List")

    def should_create_internal_cname(self):
        variables = self.get_variables()
        return all([variables["InternalZoneId"],
                    variables["InternalZoneName"],
                    variables["InternalHostname"]])

    def create_dns_records(self):
        t = self.template
        variables = self.get_variables()
        primary_endpoint = self.get_primary_address()

        if self.should_create_internal_cname():
            t.add_resource(
                RecordSetType(
                    DNS_RECORD,
                    HostedZoneId=variables["InternalZoneId"],
                    Comment="ReplicationGroup CNAME Record",
                    Name=Join(".", [variables["InternalHostname"],
                              variables["InternalZoneName"]]),
                    Type="CNAME",
                    TTL="120",
                    ResourceRecords=[primary_endpoint]))

    def create_cluster_outputs(self):
        t = self.template
        t.add_output(Output("PrimaryAddress",
                            Value=self.get_primary_address()))
        t.add_output(Output("ReadAddresses",
                            Value=Join(",", self.get_secondary_addresses())))

        t.add_output(Output("ClusterPort",
                            Value=GetAtt(REPLICATION_GROUP,
                                         "PrimaryEndPoint.Port")))
        t.add_output(Output("ClusterId", Value=Ref(REPLICATION_GROUP)))
        if self.should_create_internal_cname():
            t.add_output(
                Output(
                    "PrimaryCname",
                    Value=Ref(DNS_RECORD)))

    def create_template(self):
        self.create_parameter_group()
        self.create_subnet_group()
        self.create_security_group()
        self.create_replication_group()
        self.create_dns_records()
        self.create_cluster_outputs()



# Bastion Stack
#
# This stack configures our bastion host(s).
# http://en.wikipedia.org/wiki/Bastion_host
#
# These hosts are the only SSH entrypoint into the VPC. To SSH to a host inside
# the VPC you must first SSH to a bastion host, and then SSH from that host to
# another inside the VPC.

from troposphere import Ref, ec2, autoscaling, FindInMap, Output
from troposphere.autoscaling import Tag as ASTag

from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import (
    CFNCommaDelimitedList,
    CFNNumber,
    CFNString,
    EC2KeyPairKeyName,
    EC2SecurityGroupId,
    EC2SubnetIdList,
    EC2VPCId,
)

CLUSTER_SG_NAME = "BastionSecurityGroup"


class Bastion(Blueprint):
    VARIABLES = {
        "VpcId": {"type": EC2VPCId, "description": "Vpc Id"},
        "DefaultSG": {"type": EC2SecurityGroupId,
                      "description": "Top level security group."},
        "PublicSubnets": {"type": EC2SubnetIdList,
                          "description": "Subnets to deploy public "
                                         "instances in."},
        "PrivateSubnets": {"type": EC2SubnetIdList,
                           "description": "Subnets to deploy private "
                                          "instances in."},
        "AvailabilityZones": {"type": CFNCommaDelimitedList,
                              "description": "Availability Zones to deploy "
                                             "instances in."},
        "InstanceType": {"type": CFNString,
                         "description": "EC2 Instance Type",
                         "default": "m3.medium"},
        "MinSize": {"type": CFNNumber,
                    "description": "Minimum # of instances.",
                    "default": "1"},
        "MaxSize": {"type": CFNNumber,
                    "description": "Maximum # of instances.",
                    "default": "5"},
        "SshKeyName": {"type": EC2KeyPairKeyName},
        "OfficeNetwork": {
            "type": CFNString,
            "description": "CIDR block allowed to connect to bastion hosts."},
        "ImageName": {
            "type": CFNString,
            "description": "The image name to use from the AMIMap (usually "
                           "found in the config file.)",
            "default": "bastion"},
    }

    def create_security_groups(self):
        t = self.template
        cluster_rules = []
        cluster_rules.append(
            ec2.SecurityGroupRule(IpProtocol='tcp',
                                  FromPort=22, ToPort=22,
                                  CidrIp=Ref('OfficeNetwork')))
        sg = t.add_resource(
            ec2.SecurityGroup(CLUSTER_SG_NAME,
                              GroupDescription='BastionSecurityGroup',
                              SecurityGroupIngress=cluster_rules,
                              VpcId=Ref("VpcId")))

        t.add_output(
            Output(
                'SecurityGroup',
                Value=Ref(sg)
            )
        )

        # Make it so the bastion hosts can ssh into any other host.
        t.add_resource(
            ec2.SecurityGroupIngress(
                'AllowSSHAnywhere',
                IpProtocol='tcp',
                FromPort=22,
                ToPort=22,
                SourceSecurityGroupId=Ref(CLUSTER_SG_NAME),
                GroupId=Ref('DefaultSG')))

    def create_autoscaling_group(self):
        t = self.template
        t.add_resource(
            autoscaling.LaunchConfiguration(
                'BastionLaunchConfig',
                AssociatePublicIpAddress=True,
                ImageId=FindInMap(
                    'AmiMap', Ref("AWS::Region"), Ref("ImageName")),
                InstanceType=Ref("InstanceType"),
                KeyName=Ref("SshKeyName"),
                UserData=self.generate_user_data(),
                SecurityGroups=[Ref("DefaultSG"), Ref(CLUSTER_SG_NAME)]))
        t.add_resource(
            autoscaling.AutoScalingGroup(
                'BastionAutoscalingGroup',
                AvailabilityZones=Ref("AvailabilityZones"),
                LaunchConfigurationName=Ref("BastionLaunchConfig"),
                MinSize=Ref("MinSize"),
                MaxSize=Ref("MaxSize"),
                VPCZoneIdentifier=Ref("PublicSubnets"),
                Tags=[ASTag('Name', 'bastion', True)]))

    def generate_user_data(self):
        return ''

    def create_template(self):
        self.create_security_groups()
        self.create_autoscaling_group()



from troposphere.ec2 import SecurityGroupIngress, SecurityGroupEgress
from stacker.blueprints.base import Blueprint

CLASS_MAP = {
    "IngressRules": SecurityGroupIngress,
    "EgressRules": SecurityGroupEgress,
}


class Rules(Blueprint):
    """Used to add Ingress/Egress rules to existing security groups.

    This blueprint uses two variables:
        IngressRules:
            A dict with keys of the virtual titles for each rule, and with the
            value being a dict of the parameters taken directly from:
                http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html
        EgressRules:
            A dict with keys of the virtual titles for each rule, and with the
            value being a dict of the parameters taken directly from:
                http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html

    An example:

    name: mySecurityRules
    class_path: stacker_blueprints.security_rules.Rules
    variables:
      IngressRules:
        All80ToWebserverGroup:
          CidrIp: 0.0.0.0/0
          FromPort: 80
          ToPort: 80
          GroupId: ${output WebserverStack::SecurityGroup}
          IpProtocol: tcp
    """

    VARIABLES = {
        "IngressRules": {
            "type": dict,
            "description": "A dict of ingress rules where the key is the "
                           "name of the rule to create, and the value is "
                           "a dictionary of keys/values based on the "
                           "attributes of the "
                           ":class:`troposphere.ec2.SecurityGroupIngress` "
                           "class.",
            "default": {},
        },
        "EgressRules": {
            "type": dict,
            "description": "A dict of ingress rules where the key is the "
                           "name of the rule to create, and the value is "
                           "a dictionary of keys/values based on the "
                           "attributes of the "
                           ":class:`troposphere.ec2.SecurityGroupEgress` "
                           "class.",
            "default": {},
        }
    }

    def create_security_rules(self):
        t = self.template
        variables = self.get_variables()
        for rule_type, rule_class in CLASS_MAP.items():
            for rule_title, rule_attrs in variables[rule_type].items():
                t.add_resource(rule_class.from_dict(rule_title, rule_attrs))

    def create_template(self):
        self.create_security_rules()



from troposphere import (
    Ref, Output
)

from stacker.blueprints.base import Blueprint
from stacker.util import load_object_from_string


class GenericResourceCreator(Blueprint):
    """ Generic Blueprint for creating a resource

    Example config - this would create a stack with a single resource in it,
    an ec2.Volume resource:

    - name: generic-resource-volume
        class_path: blueprints.generic.GenericResourceCreator
        variables:
            Class: ec2.Volume
            Output: VolumeId
            Properties:
                VolumeType: gp2
                Size: 5
                Encrypted: true
                AvailabilityZone: us-east-1b

    """

    VARIABLES = {
        'Class':
            {'type': str,
             'description': 'The troposphere class to create, '
                            'e.g.: ec2.Volume'},
        'Output':
            {'type': str,
             'description': 'The output field that should be created, '
                            'e.g.: VolumeId'},
        'Properties':
            {'type': dict,
             'description': 'The list of properties to use for the '
                            'Troposphere class'},
    }

    def add_cfn_description(self):
        """ Boilerplate for CFN Template


        *** NOTE ***    Template Version Reminder

        Make Sure you bump up the template version number above if submitting
        updates to the repo. This is the only way we can tell which version of
        a template is in place on a running resouce.
        """
        template = self.template
        template.add_version('2010-09-09')
        template.add_description('Generic Resource Creator - 1.0.0')

    def setup_resource(self):
        """ Setting Up Resource """
        template = self.template
        variables = self.get_variables()

        tclass = variables['Class']
        tprops = variables['Properties']
        output = variables['Output']

        klass = load_object_from_string('troposphere.' + tclass)

        instance = klass.from_dict('ResourceRefName', tprops)

        template.add_resource(instance)
        template.add_output(Output(
            output,
            Description="A reference to the object created in this blueprint",
            Value=Ref(instance)
        ))

    def create_template(self):
        """ Create the CFN template """
        self.add_cfn_description()
        self.setup_resource()



import os
from setuptools import setup, find_packages

src_dir = os.path.dirname(__file__)

install_requires = [
    "stacker~=1.0.1",
    "troposphere~=1.9.3",
]

tests_require = [
    "nose~=1.0",
    "mock~=2.0.0",
]


def read(filename):
    full_path = os.path.join(src_dir, filename)
    with open(full_path) as fd:
        return fd.read()


if __name__ == "__main__":
    setup(
        name="stacker_blueprints",
        version="1.0.3",
        author="Michael Barrett",
        author_email="loki77@gmail.com",
        license="New BSD license",
        url="https://github.com/remind101/stacker_blueprints",
        description="Default blueprints for stacker",
        long_description=read("README.rst"),
        packages=find_packages(),
        install_requires=install_requires,
        tests_require=tests_require,
        test_suite="nose.collector",
    )



from stacker.blueprints.base import Blueprint
from stacker.blueprints.variables.types import TroposphereType

from troposphere import (
    dynamodb,
    Ref,
    GetAtt,
    Output,
)


class DynamoDB(Blueprint):
    """Manages the creation of DynamoDB tables.

    Example::

      - name: users
        class_path: stacker_blueprints.dynamodb.DynamoDB
        variables:
          Tables:
            UserTable:
              TableName: prod-user-table
              KeySchema:
                - AttributeName: id
                  KeyType: HASH
                - AttributeName: name
                  KeyType: RANGE
              AttributeDefinitions:
                - AttributeName: id
                  AttributeType: S
                - AttributeName: name
                  AttributeType: S
              ProvisionedThroughput:
                ReadCapacityUnits: 5
                WriteCapacityUnits: 5
              StreamSpecification:
                StreamViewType: ALL

    """

    VARIABLES = {
        "Tables": {
            "type": TroposphereType(dynamodb.Table, many=True),
            "description": "DynamoDB tables to create.",
        }
    }

    def create_template(self):
        t = self.template
        variables = self.get_variables()
        for table in variables["Tables"]:
            t.add_resource(table)
            stream_enabled = table.properties.get("StreamSpecification")
            if stream_enabled:
                t.add_output(Output("{}StreamArn".format(table.title),
                                    Value=GetAtt(table, "StreamArn")))
            t.add_output(Output("{}Name".format(table.title),
                                Value=Ref(table)))



from stacker.blueprints.base import Blueprint

from troposphere import (
    sns,
    sqs,
    Ref,
    GetAtt,
    Output,
)

from . import util

import awacs
import awacs.sqs

from awacs.aws import (
    Policy,
    Statement,
    Condition,
    ArnEquals,
    Principal,
)


def queue_policy(sns_arn, sqs_arns):
    stmts = []
    for arn in sqs_arns:
        stmts.append(
            Statement(
                Effect="Allow",
                Principal=Principal("*"),
                Action=[awacs.sqs.SendMessage],
                Resource=[arn],
                Condition=Condition(
                    ArnEquals({"aws:SourceArn": sns_arn})
                )
            )
        )

    return Policy(Statement=stmts)


def validate_topic(topic):
    sns_topic_properties = [
        "DisplayName",
        "Subscription",
    ]

    util.check_properties(topic, sns_topic_properties, "SNS")

    return topic


def validate_topics(topics):
    validated_topics = {}
    for topic_name, topic_config in topics.iteritems():
        validated_topics[topic_name] = validate_topic(topic_config)

    return validated_topics


class Topics(Blueprint):
    """
    Manages the creation of SNS topics.
    """

    VARIABLES = {
        "Topics": {
            "type": dict,
            "description": "Dictionary of SNS Topic definitions",
            "validator": validate_topics,
        }
    }

    def create_template(self):
        variables = self.get_variables()

        for topic_name, topic_config in variables["Topics"].iteritems():
            self.create_topic(topic_name, topic_config)

    def create_sqs_policy(self, topic_name, topic_arn, topic_subs):
        """
        This method creates the SQS policy needed for an SNS subscription. It
        also takes the ARN of the SQS queue and converts it to the URL needed
        for the subscription, as that takes a URL rather than the ARN.
        """
        t = self.template

        arn_endpoints = []
        url_endpoints = []
        for sub in topic_subs:
            arn_endpoints.append(sub["Endpoint"])
            split_endpoint = sub["Endpoint"].split(":")
            queue_url = "https://%s.%s.amazonaws.com/%s/%s" % (
                split_endpoint[2],  # literally "sqs"
                split_endpoint[3],  # AWS region
                split_endpoint[4],  # AWS ID
                split_endpoint[5],  # Queue name
            )
            url_endpoints.append(queue_url)

        policy_doc = queue_policy(topic_arn, arn_endpoints)

        t.add_resource(
            sqs.QueuePolicy(
                topic_name + "SubPolicy",
                PolicyDocument=policy_doc,
                Queues=url_endpoints,
            )
        )

    def create_topic(self, topic_name, topic_config):
        """
        Creates the SNS topic, along with any subscriptions requested.
        """
        topic_subs = None
        t = self.template

        if "Subscription" in topic_config:
            topic_subs = topic_config["Subscription"]

        t.add_resource(
            sns.Topic.from_dict(
                topic_name,
                topic_config
            )
        )

        topic_arn = Ref(topic_name)

        t.add_output(
            Output(topic_name + "Name", Value=GetAtt(topic_name, "TopicName"))
        )
        t.add_output(Output(topic_name + "Arn", Value=topic_arn))

        if topic_subs:
            self.create_sqs_policy(topic_name, topic_arn, topic_subs)
